/* eslint-disable max-len */
import React from 'react';
import PropTypes from 'prop-types';
// import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import { Card } from 'primereact/card';
import untar from 'js-untar';
import moment from 'moment';
import xml2js from 'xml2js';
import filesize from 'filesize';
import sha256 from 'crypto-js/sha256';
import encHex from 'crypto-js/enc-hex';
import encB64 from 'crypto-js/enc-base64';
import typedarrays from 'crypto-js/lib-typedarrays';
import { TextDecoder } from 'text-encoding';
import Encoding from 'encoding-japanese';
import CustomMessages from '../components/CustomMessages';
import BaseContainer from './BaseContainer';
import ActionButton from '../components/ActionButton';
import ActionLink from '../components/ActionLink';
import DivContainer from '../components/DivContainer';
import CustomLabel from '../components/CustomLabel';
import InputFileChooseComponent from '../components/inputs/InputFileChooseComponent';
import InputFileUploadComponent from '../components/inputs/InputFileUploadComponent';
import InputCalendarComponent from '../components/inputs/InputCalendarComponent';
import InputCalendarRangeComponent from '../components/inputs/InputCalendarRangeComponent';
import InputCheckboxComponent from '../components/inputs/InputCheckboxComponent';
import InputMultiCheckboxComponent from '../components/inputs/InputMultiCheckboxComponent';
import InputSelectButtonComponent from '../components/inputs/InputSelectButtonComponent';
import InputDropdownComponent from '../components/inputs/InputDropdownComponent';
import EditableDataTableExt from '../components/EditableDataTableExt';
import CustomDataTable from '../components/base/CustomDataTable';
import SimpleReactValidator from '../validator';
import BaseService from './BaseService';
import {
	verifySignature,
	NoSzafirExtDialog,
	NoSzafirHostDialog,
} from '../components/validation/VerifySignature';

class ValidationStandaloneService extends BaseService {
	// Initializing important variables
	constructor(domain, path) {
		super(domain);
		this.path = path;
	}

	validateXmls(xmlList) {
		const data = new FormData();
		xmlList.forEach(xml => {
			data.append('xmlFile', JSON.stringify(xml).replace('+', '&plus;'));
		});
		return this.fetch(
			`${this.domain}/${this.path}/validation/validate-xml-list`,
			{
				method: 'POST',
				body: data,
			},
			{
				Accept: 'application/json',
				'Content-Type': 'multipart/form-data',
			}
		).then(res => {
			return Promise.resolve(res);
		});
	}

	uploadAndCreateFile(data) {
		return this.fetch(`${this.domain}/${this.path}/validation/upload-and-create-file`, {
			method: 'POST',
			body: JSON.stringify(data),
		}).then(res => {
			return Promise.resolve(res);
		});
	}
}

class BaseValidationStandaloneContainer extends BaseContainer {
	constructor(props, path) {
		super(props);
		this.service = new ValidationStandaloneService(props.baseRestApiUrl, path);
		this.service.setUiMethods(this.blockUi, this.unblockUi);
		this.path = path;
		this.state = {
			loading: true,
			extractedFiles: [],
			parsedFiles: [],
			contentList: {
				contentListFile: { contentListRows: [] },
				attachment: undefined,
				fileName: 'spis-zo.xml',
				createDate: undefined,
				dateFrom: undefined,
				dateTo: undefined,
			},
			requestValidationTypeOptions: [
				{ value: 'SIGNATURE', label: 'Pieczęć elektroniczna/podpis' },
				{ value: 'CHECKSUM', label: 'Suma kontrolna' },
				{ value: 'CONTENT_LIST', label: 'Porównanie ze spisem zd-odb' },
				{ value: 'STRUCTURE_COMPARE', label: 'Porównanie ze strukturą i metadanymi' },
				{ value: 'ENTIRE_PACKAGE_VALIDATION', label: 'Walidacja całkowita paczki' },
			],
			contentListTypeOptions: [
				{ value: 'ATTACH', label: 'Załącz' },
				{ value: 'CREATE', label: 'Utwórz' },
			],
			requestValidationTypes: [],
			autoVerified: false,
			structureVerificationInProgress: false,
			signatureVerificationInProgress: false,
			entirePackageValidationInProgress: false,
			contentListVerificationInProgress: true,
			xsdFiles: [],
			reportFormatTypeOptions: [{ value: 'PDF', label: '.pdf' }, { value: 'XML', label: '.xml' }],
			contentListDetailsToReport: undefined,
		};
		this.parseFiles = this.parseFiles.bind(this);
		this.runVerification = this.runVerification.bind(this);
		this.handleAfterFilesChange = this.handleAfterFilesChange.bind(this);
		this.handleAfterChecksumFilesChange = this.handleAfterChecksumFilesChange.bind(this);
		this.handleAfterSignatureFilesChange = this.handleAfterSignatureFilesChange.bind(this);
		this.handleAfterContentListFilesChange = this.handleAfterContentListFilesChange.bind(this);
		this.handleAfterRemoveChecksum = this.handleAfterRemoveChecksum.bind(this);
		this.handleAfterRemoveContentList = this.handleAfterRemoveContentList.bind(this);
		this.handleAfterRemoveSignature = this.handleAfterRemoveSignature.bind(this);
		this.fileTemplate = this.fileTemplate.bind(this);
		this.renderContentListAttachmentInfo = this.renderContentListAttachmentInfo.bind(this);
		this.autoVerification = this.autoVerification.bind(this);
		this.verifyChecksum = this.verifyChecksum.bind(this);
		this.verifySignatureTask = this.verifySignatureTask.bind(this);
		this.verifyStructure = this.verifyStructure.bind(this);
		this.verifyContentList = this.verifyContentList.bind(this);
		this.verifyPackageWithContentList = this.verifyPackageWithContentList.bind(this);
		this.validationChecked = this.validationChecked.bind(this);
		this.onVerificationError = this.onVerificationError.bind(this);
		this.onSignatureVerified = this.onSignatureVerified.bind(this);
		this.entirePackageValidation = this.entirePackageValidation.bind(this);
		this.entirePackageValidationStep1Success = this.entirePackageValidationStep1Success.bind(this);
		this.entirePackageValidationStep1Error = this.entirePackageValidationStep1Error.bind(this);
		this.entirePackageValidationStep2 = this.entirePackageValidationStep2.bind(this);
		this.contentListValidator = new SimpleReactValidator();
		this.formatTypeValidator = new SimpleReactValidator();
		this.signatureValidationReportFormatTypeValidator = new SimpleReactValidator();
		this.checksumValidationReportFormatTypeValidator = new SimpleReactValidator();
		this.handoverListValidationReportFormatTypeValidator = new SimpleReactValidator();
		this.schemaValidationReportFormatTypeValidator = new SimpleReactValidator();
		this.entirePackageValidationReportFormatTypeValidator = new SimpleReactValidator();
		this.signatureTimeout = null;
	}

	componentDidMount() {
		super.componentDidMount();
		if (!window.TextDecoder) {
			window.TextDecoder = TextDecoder;
		}
	}

	validationChecked(validationType) {
		let result = false;
		if (
			this.state.requestValidationTypes !== undefined &&
			this.state.requestValidationTypes !== null
		) {
			this.state.requestValidationTypes.forEach(type => {
				if (type.value === validationType) {
					result = true;
				}
			});
		}
		return result;
	}

	authorTemplate(node) {
		if (
			node.data !== undefined &&
			node.data.fileInfo !== undefined &&
			node.data.fileInfo.author !== undefined
		) {
			return node.data.fileInfo.author;
		} else {
			return '';
		}
	}

	createDateTemplate(format, node) {
		if (
			node.data !== undefined &&
			node.data.fileInfo !== undefined &&
			node.data.fileInfo.createDate !== undefined &&
			moment(node.data.fileInfo.createDate).isValid()
		) {
			return moment(node.data.fileInfo.createDate).format(format);
		} else {
			return '';
		}
	}

	verifyChecksum() {
		const result = {
			validationType: {
				value: 'CHECKSUM_VALIDATION',
				label: 'Weryfikacja sumy kontrolnej',
			},
			success: true,
			errors: [],
		};
		const packageCheckSumHexString = new String(this.state.packageCheckSumHex).toUpperCase().trim();
		const packageCheckSumB64String = new String(this.state.packageCheckSumB64).toUpperCase().trim();
		const uploadedCheckSumString = new String(this.state.uploadedCheckSum).toUpperCase().trim();
		if (this.state.uploadedCheckSum === undefined || this.state.uploadedCheckSum === null) {
			result.success = false;
			result.errors.push({
				errorMsg: 'Nie można odczytać skrótu z załączonego pliku.',
			});
		} else if (
			packageCheckSumHexString !== uploadedCheckSumString &&
			packageCheckSumB64String !== uploadedCheckSumString
		) {
			result.success = false;
			result.errors.push({
				errorMsg: 'Skrót z pliku nie zgadza się z obliczonym skrótem dla paczki archiwalnej.',
			});
		}
		return result;
	}

	verifySignatureTask() {
		this.setState({ signatureVerificationInProgress: true }, () => {
			verifySignature(
				this.state.packageTar,
				this.state.signature,
				this.onSignatureVerified,
				this.onVerificationError,
				false,
				false,
				'http://localhost:8088/'
			);
		});
	}

	onVerificationError(error) {
		// clearTimeout(this.signatureTimeout);
		let visibleNoSzafirExtDialog = false;
		const result = {
			validationType: {
				value: 'SIGNATURE_VALIDATION',
				label: 'Weryfikacja podpisu',
			},
			success: false,
			errors: [],
		};
		if (error === 'noExtension') {
			visibleNoSzafirExtDialog = true;
			result.errors.push({
				errorMsg: 'Brak rozszerzenia Szafir SDK',
			});
		} else if (error === 'noHostApp') {
			visibleNoSzafirExtDialog = true;
			result.errors.push({
				errorMsg: 'Brak aplikacji Szafir Host',
			});
		} else if (error === 'timeout') {
			result.errors.push({
				errorMsg: 'Przekroczono czas oczekiwania na wynik weryfikacji. Proszę spróbować ponownie.',
			});
		} else {
			result.errors.push({
				errorMsg: 'Nieprawidłowy skrót pliku z podpisem',
			});
		}
		this.setState(
			{
				signatureReport: result,
				signatureVerificationInProgress: false,
				visibleNoSzafirExtDialog,
			},
			() => {
				// this.state.requestValidationTypes.forEach(validationType => {
				// 	if (validationType.value === 'ENTIRE_PACKAGE_VALIDATION') {
				// 		this.entirePackageValidation();
				// 	}
				// });
			}
		);
	}

	onSignatureVerified(result) {
		// clearTimeout(this.signatureTimeout);
		const successObj = {
			signerName: result.subject,
			signDate: result.signDate,
			certId: result.certId,
			certIssuer: result.issuer,
		};
		const report = {
			validationType: {
				value: 'SIGNATURE_VALIDATION',
				label: 'Weryfikacja podpisu',
			},
			success: result.code === 0 || result.code === '0',
			errors: [],
			successDetails: JSON.stringify(successObj).toString(),
		};
		if (result.code !== 0 && result.code !== '0') {
			report.errors.push({
				errorMsg: result.desc !== undefined ? result.desc : result.codeDesc,
			});
		}
		this.setState({ signatureReport: report, signatureVerificationInProgress: false }, () => {
			// this.state.requestValidationTypes.forEach(validationType => {
			// 	if (validationType.value === 'ENTIRE_PACKAGE_VALIDATION') {
			// 		this.entirePackageValidation();
			// 	}
			// });
		});
	}

	verifyContentList(entireValidation) {
		if (!entireValidation) {
			this.setState({ contentListVerificationInProgress: true });
		}
		return new Promise((resolve, reject) => {
			if (
				this.state.contentListType !== undefined &&
				this.state.contentListType.value === 'ATTACH' &&
				this.state.contentList.file !== undefined
			) {
				let contentListXml;
				let invalidFileFormat = false;
				xml2js.parseString(this.state.contentList.file, { explicitArray: false }, (err, result) => {
					if (err !== undefined && err !== null) {
						invalidFileFormat = true;
					} else {
						contentListXml = result;
					}
				});
				if (invalidFileFormat) {
					const caseErrors = [];
					const caseFilesInPackageMissingInHandoverList = [];
					this.state.extractedFiles.forEach(extractedFile => {
						const fileDirectory = extractedFile.name.split(/[\/\\]/);
						if (!extractedFile.name.includes('@LongLink')) {
							const index = this.checkParentFolderIndex(fileDirectory);
							if (
								fileDirectory[index] !== 'dokumenty' &&
								fileDirectory[index] !== 'sprawy' &&
								fileDirectory[index] !== 'metadane'
							) {
							}
							switch (fileDirectory[index]) {
								case 'sprawy':
									if (extractedFile.type === '0') {
										const caseFileName = fileDirectory[fileDirectory.length - 1].replace(
											'.xml',
											''
										);
										if (caseFileName) {
											caseErrors.push({
												errorMsg: `W spisie zd-odbiorczym brakuje sprawy, która jest w folderze sprawy: ${caseFileName}`,
											});
											caseFilesInPackageMissingInHandoverList.push(caseFileName);
										}
									}
									break;
								default:
									break;
							}
						}
					});
					const hoErrors = [
						{
							errorMsg: 'Wystąpił błąd przy odczycie spisu zdawczo-odbiorczego',
						},
					];
					return reject({
						validationType: {
							value: 'HANDOVER_LIST_VALIDATION',
							label: 'Weryfikacja spisu zdawczo-odbiorczego',
						},
						success: false,
						errors: hoErrors.concat(caseErrors),
						successDetails: JSON.stringify({ caseFilesInPackageMissingInHandoverList }),
					});
				}
				if (
					contentListXml !== undefined &&
					contentListXml.SpisZdawczoOdbiorczy !== undefined &&
					contentListXml.SpisZdawczoOdbiorczy !== null &&
					contentListXml.SpisZdawczoOdbiorczy.Sprawa !== undefined &&
					contentListXml.SpisZdawczoOdbiorczy.Sprawa !== null &&
					contentListXml.SpisZdawczoOdbiorczy.Sprawa instanceof Array &&
					contentListXml.SpisZdawczoOdbiorczy.Sprawa.length > 0
				) {
					return Promise.resolve(
						this.verifyPackageWithContentList(contentListXml.SpisZdawczoOdbiorczy.Sprawa, resolve)
					);
				} else if (
					contentListXml !== undefined &&
					contentListXml.SpisZdawczoOdbiorczy !== undefined &&
					contentListXml.SpisZdawczoOdbiorczy !== null &&
					contentListXml.SpisZdawczoOdbiorczy.Sprawa !== undefined &&
					contentListXml.SpisZdawczoOdbiorczy.Sprawa !== null &&
					contentListXml.SpisZdawczoOdbiorczy.Sprawa instanceof Object
				) {
					return Promise.resolve(
						this.verifyPackageWithContentList([contentListXml.SpisZdawczoOdbiorczy.Sprawa], resolve)
					);
				} else {
					return {
						validationType: {
							value: 'HANDOVER_LIST_VALIDATION',
							label: 'Weryfikacja spisu zdawczo-odbiorczego',
						},
						success: false,
						errors: [
							{
								errorMsg: 'Brak spraw w spisie zdawczo-odbiorczym.',
							},
						],
					};
				}
			} else if (
				this.state.contentListType !== undefined &&
				this.state.contentListType.value === 'CREATE'
			) {
				const spisZdawczoOdbiorczy = [];
				this.state.contentList.contentListFile.contentListRows.forEach(row => {
					spisZdawczoOdbiorczy.push({
						DataWszczecia: moment(row.caseBeginDate).format('YYYY-MM-DD'),
						DataZakonczenia: moment(row.caseEndDate).format('YYYY-MM-DD'),
						HasloKlasyfikacyjne: row.classificationEntry,
						LiczbaDokumentow: row.documentCount,
						LiczbaPorzadkowa: row.id,
						TytulSprawy: row.caseTitle,
						ZnakSprawy: row.caseNumber,
					});
				});

				return Promise.resolve(this.verifyPackageWithContentList(spisZdawczoOdbiorczy, resolve));
			}
		});
	}

	verifyPackageWithContentList(casesFromFile, resolve) {
		const cases = {};
		casesFromFile.forEach(c => {
			cases[c.ZnakSprawy] = c;
		});
		const result = {
			validationType: {
				value: 'HANDOVER_LIST_VALIDATION',
				label: 'Weryfikacja spisu zdawczo-odbiorczego',
			},
			success: true,
			errors: [],
		};
		const filesFromSprawyDirectory = [];
		const caseFilesInPackageMissingInHandoverList = [];
		const handoverListCaseMissingInFiles = [];
		const casesSuccessObject = [];
		this.state.extractedFiles.forEach(extractedFile => {
			const fileDirectory = extractedFile.name.split(/[\/\\]/);
			if (!extractedFile.name.includes('@LongLink')) {
				const index = this.checkParentFolderIndex(fileDirectory);
				if (
					fileDirectory[index] !== 'dokumenty' &&
					fileDirectory[index] !== 'sprawy' &&
					fileDirectory[index] !== 'metadane'
				) {
				}
				switch (fileDirectory[index]) {
					case 'sprawy':
						if (extractedFile.type === '0') {
							const caseFileName = fileDirectory[fileDirectory.length - 1].replace('.xml', '');
							if (caseFileName) {
								filesFromSprawyDirectory.push(caseFileName);
								// todo regex xml
								// const expectedCaseId = caseFileName.replace('.xml', '');
								if (!cases[caseFileName]) {
									result.success = false;
									result.errors.push({
										errorMsg: `W spisie zd-odbiorczym brakuje sprawy, która jest w folderze sprawy: ${caseFileName}`,
									});
									caseFilesInPackageMissingInHandoverList.push(caseFileName);
								}
								const content = new TextDecoder().decode(new Uint8Array(extractedFile.buffer));
								let xmlContent;
								xml2js.parseString(content, { explicitArray: false }, (err, result) => {
									if (err !== undefined && err !== null) {
										const errorMsg = `Plik sprawy nie jest poprawnym dokumentem XML ${extractedFile.extractedName}`;
										if (
											result &&
											result.errors &&
											result.errors.filter(e => e.errorMsg === errorMsg).length <= 0
										) {
											result.success = false;
											result.errors.push({
												errorMsg,
											});
										}
									}
									xmlContent = result;
								});
								if (xmlContent !== undefined && xmlContent !== null) {
									if (
										xmlContent.dokument &&
										xmlContent.dokument &&
										xmlContent.dokument.identyfikator &&
										xmlContent.dokument.identyfikator instanceof Array
									) {
										xmlContent.dokument.identyfikator.forEach(g => {
											if (g.typ === 'znak sprawy') {
												casesSuccessObject.push(g.wartosc);
											}
										});
									} else {
										if (
											xmlContent.dokument &&
											xmlContent.dokument.identyfikator &&
											xmlContent.dokument.identyfikator.typ === 'znak sprawy'
										) {
											casesSuccessObject.push(xmlContent.dokument.identyfikator.wartosc);
										}
									}
								}
							}
						}
						break;
					default:
						break;
				}
			}
		});

		if (casesFromFile === undefined || casesFromFile.length === 0) {
			result.success = false;
			result.errors.push({
				errorMsg: 'Spis zdawczo-odbiorczy jest pusty.',
			});
		}

		const casesFromSprawyDirectory = [];
		this.state.extractedFiles.forEach(c => {
			if (c.fileType && c.fileType === 'Sprawa') {
				casesFromSprawyDirectory.push(c);
			}
		});
		if (casesSuccessObject.length !== casesFromFile.length) {
			result.success = false;
			result.errors.push({
				errorMsg:
					'Ilość spraw z listy zdawczo-zbiorczej nie zgadza się z danymi z paczki archiwalnej',
			});
		}
		const parsedCases = [];
		this.state.parsedFiles.forEach(parsedFile => {
			// BARDZO WAŻNE !
			if (parsedFile.data && parsedFile.data.fileInfo && parsedFile.data.fileInfo.title) {
				parsedCases[parsedFile.data.extractedName] = parsedFile;
			} else {
				parsedCases[
					parsedFile.data.extractedName
						.substring(0, parsedFile.data.extractedName.length - 4)
						.toUpperCase()
				] = parsedFile;
			}
		});
		const casesToShowInSuccessReport = [];

		casesFromFile.forEach(caseFromContentList => {
			const caseFile = parsedCases[caseFromContentList.ZnakSprawy];
			const caseFileMaybeList = casesSuccessObject.filter(c => {
				return c === caseFromContentList.ZnakSprawy;
			});
			if (caseFileMaybeList && caseFileMaybeList.length && caseFileMaybeList.length > 1) {
				caseFileMaybeList.forEach(c => {
					result.success = false;
					result.errors.push({
						errorMsg: `Znaleziono więcej niż jedną sprawę z takim samym numerem sprawy ${c} w paczce archiwalnej`,
					});
				});
			}
			if (
				!caseFile ||
				(this.state.parsedFiles && this.state.parsedFiles.length === 0) ||
				!filesFromSprawyDirectory.includes(caseFile.data.extractedName)
			) {
				handoverListCaseMissingInFiles.push(`${caseFromContentList.ZnakSprawy}`);
				result.success = false;
				result.errors.push({
					errorMsg: `W folderze sprawy brakuje pliku sprawy, która jest obecna w spisie zdawczo-odbiorczym: ${caseFromContentList.ZnakSprawy}`,
				});
			}
			if (caseFile === undefined || caseFile === null) {
				result.success = false;
				result.errors.push({
					errorMsg: `Nie można znaleźć sprawy z spisu zdawczo-odbiorczego ze znakiem sprawy ${caseFromContentList.ZnakSprawy} w paczce archiwalnej.`,
				});
			} else {
				if (
					caseFile.data.fileInfo.title === undefined ||
					caseFile.data.fileInfo.title === null ||
					caseFromContentList.TytulSprawy === undefined ||
					caseFromContentList.TytulSprawy === null ||
					caseFile.data.fileInfo.title !== caseFromContentList.TytulSprawy
				) {
					result.success = false;
					result.errors.push({
						errorMsg: `Tytuł sprawy ze spisu zdawczo-odbiorczego dla sprawy ${caseFromContentList.ZnakSprawy} nie zgadza się z tym z paczki archiwalnej.`,
					});
				}
				if (
					caseFile.data.fileInfo.endDate === undefined ||
					caseFile.data.fileInfo.endDate === null ||
					caseFromContentList.DataZakonczenia === undefined ||
					caseFromContentList.DataZakonczenia === null ||
					new Date(caseFile.data.fileInfo.endDate).getTime() !==
						new Date(caseFromContentList.DataZakonczenia).getTime()
				) {
					result.success = false;
					result.errors.push({
						errorMsg: `Data zakończenia ze spisu zdawczo-odbiorczego dla sprawy ${caseFromContentList.ZnakSprawy} nie zgadza się z datą z pliku sprawy z paczki archiwalnej.`,
					});
				}
				if (
					caseFile.data.fileInfo.createDate === undefined ||
					caseFile.data.fileInfo.createDate === null ||
					caseFromContentList.DataWszczecia === undefined ||
					caseFromContentList.DataWszczecia === null ||
					new Date(caseFile.data.fileInfo.createDate).getTime() !==
						new Date(caseFromContentList.DataWszczecia).getTime()
				) {
					result.success = false;
					result.errors.push({
						errorMsg: `Data wszczęcia ze spisu zdawczo-odbiorczego dla sprawy ${caseFromContentList.ZnakSprawy} nie zgadza się z datą z pliku sprawy z paczki archiwalnej.`,
					});
				}
				let documentCount = 0;
				if (caseFile.children !== undefined && caseFile.children !== null) {
					documentCount = caseFile.children.length;
				}
				if (
					documentCount === undefined ||
					documentCount === null ||
					caseFromContentList.LiczbaDokumentow === undefined ||
					caseFromContentList.LiczbaDokumentow === null ||
					documentCount !== parseInt(caseFromContentList.LiczbaDokumentow)
				) {
					result.success = false;
					result.errors.push({
						errorMsg: `Liczba dokumentów ze spisu zdawczo-odbiorczego dla sprawy ${caseFromContentList.ZnakSprawy} nie zgadza się z liczbą dokumentów z paczki archiwalnej.`,
					});
				}
			}
			casesToShowInSuccessReport.push(caseFromContentList.ZnakSprawy);
		});
		result.successCases = casesToShowInSuccessReport;
		result.successDetails = JSON.stringify({
			caseIds: casesToShowInSuccessReport,
			caseFilesInPackageMissingInHandoverList,
			handoverListCaseMissingInFiles,
		}).toString();
		return resolve(result);
	}

	verifyStructure(isFromOrderedValidation) {
		return new Promise(resolve => {
			const result = {
				validationType: {
					value: 'SCHEMA_VALIDATION',
					label: 'Weryfikacja struktury i metadanych',
				},
				success:
					this.state.parsingErrors === undefined ||
					this.state.parsingErrors === null ||
					this.state.parsingErrors.length === 0,
				errors: this.state.parsingErrors,
			};
			let correctFolders;
			let additionalObjects = 'NIE';
			let allFilesInCaseFolderValid = true;
			if (
				this.state.parsedFiles === undefined ||
				this.state.parsedFiles === null ||
				this.state.parsedFiles.length === 0 ||
				this.state.extractedFiles === undefined ||
				this.state.extractedFiles === null ||
				this.state.extractedFiles.length === 0
			) {
				if (!isFromOrderedValidation) {
					// this.setState({
					// 	structureCompareReport: result,
					// 	structureVerificationInProgress: false,
					// });
				}
				result.success = false;
				result.errors.push({
					errorMsg:
						'Struktura głównych katalogów w paczce jest niepoprawna! Nie można zwalidować poszczególnych plików paczki archiwalnej.',
				});
				correctFolders = 'NIE';
				// result.successDetails = JSON.stringify({
				// 	correctFolders,
				// 	correctFileNames: 'NIE',
				// }).toString();
				// return result;
			}
			let documentsFound = false;
			let casesFound = false;
			let metadataFound = false;
			let otherFound = false;
			const casesSuccessObject = [];
			const casesToPrint = [];
			const documentSuccessObject = [];
			const metadataFileNamesList = [];
			let invalidFileNameFound = false;
			const restricted = ['*', '?', ':', '=', ',', ';', '/', '\\'];
			const folderNames = [];
			const folders = [];
			let emptyFolders;
			let casesNotOnlyXml = 'NIE';
			const metadatasWithoutDocument = [];
			const documentsWithoutMetadata = [];

			// let metadatasWithMissingTags = [];
			// let incorrectMetadataXsdName = [];
			// let casesWithMissingTags = [];
			// let documentsCount = 0;
			// let metadataCount = 0;
			const caseFileNames = [];
			const caseFilesWithNoMetadataAssigment = [];
			const fileNamesWithNoGroupingTag = [];
			const documentsFileNames = [];
			let metadataAndDocumentSameCount = 'TAK';
			let metadataDirPresent = true;
			let caseDirPresent = true;
			let packageFileNamesInUtf8Encoding = true;
			let metadatasOver = false;
			// const fileNamesInSprawyDirectory = [];
			this.state.extractedFiles.forEach(extractedFile => {
				if (!extractedFile.name.includes('@LongLink')) {
					const fileDirectory = extractedFile.name.split(/[\/\\]/);
					if (
						fileDirectory[0] === 'dokumenty' &&
						fileDirectory.length > 2 &&
						extractedFile.type === '5'
					) {
						folderNames.push(fileDirectory[fileDirectory.length - 2]);
						folders[fileDirectory[fileDirectory.length - 2]] = 0;
					}
					if (
						!extractedFile.name.match(
							'^[\\węóąśłżźćńĘÓĄŚŁŻŹĆŃ\\!@#\\$%\\^&\\(\\)_\\+=\\-\\*\\.,\\ \\/]+$'
						)
					) {
						packageFileNamesInUtf8Encoding = false;
					}
				}
			});

			const metadatas = [];
			const caseIdsFromDocumentsTag = [];
			const filesWithNOUtf8Encoding = [];
			this.state.extractedFiles.forEach(extractedFile => {
				const fileDirectory = extractedFile.name.split(/[\/\\]/);
				if (!extractedFile.name.includes('@LongLink')) {
					const index = this.checkParentFolderIndex(fileDirectory);
					if (
						fileDirectory[index] === 'dokumenty' ||
						fileDirectory[index] === 'sprawy' ||
						fileDirectory[index] === 'metadane'
					) {
					}
					switch (fileDirectory[index]) {
						case 'dokumenty':
							if (fileDirectory.length > 2) {
								for (let i = 1; i < fileDirectory.length - 1; i++) {
									if (
										folders[fileDirectory[i]] !== undefined &&
										folders[fileDirectory[i]] !== null
									) {
										folders[fileDirectory[i]] = folders[fileDirectory[i]] + 1;
									}
								}
							}
							if (extractedFile.type === '0') {
								documentsFound = true;
								// documentsCount += 1;
								documentSuccessObject.push(fileDirectory[fileDirectory.length - 1]);
								if (
									fileDirectory[fileDirectory.length - 1] &&
									fileDirectory[fileDirectory.length - 1] !== ''
								) {
									documentsFileNames.push(`dokumenty/${fileDirectory[fileDirectory.length - 1]}`);
								}
							}
							break;
						case 'metadane':
							if (extractedFile.type === '0') {
								metadataFound = true;
								metadataFileNamesList.push(fileDirectory[fileDirectory.length - 1]);

								extractedFile.extractedName = fileDirectory[fileDirectory.length - 1];
								extractedFile.fileType = 'Metadane';
								metadatas.push(extractedFile);
							}
							break;
						case 'sprawy':
							if (extractedFile.type === '0') {
								casesFound = true;
								const content = new TextDecoder().decode(new Uint8Array(extractedFile.buffer));
								let xmlContent;
								xml2js.parseString(content, { explicitArray: false }, (err, result) => {
									if (err !== undefined && err !== null) {
										const errorMsg = `Plik sprawy nie jest poprawnym dokumentem XML ${extractedFile.extractedName}`;
										if (
											result &&
											result.errors &&
											result.errors.filter(e => e.errorMsg === errorMsg).length <= 0
										) {
											result.success = false;
											result.errors.push({
												errorMsg,
											});
										}
									}
									xmlContent = result;
								});
								if (xmlContent !== undefined && xmlContent !== null) {
									if (
										xmlContent.dokument &&
										xmlContent.dokument &&
										xmlContent.dokument.identyfikator &&
										xmlContent.dokument.identyfikator instanceof Array
									) {
										xmlContent.dokument.identyfikator.forEach(g => {
											if (g.typ === 'znak sprawy') {
												casesSuccessObject.push(g.wartosc);
											}
										});
									} else {
										if (
											xmlContent.dokument &&
											xmlContent.dokument.identyfikator &&
											xmlContent.dokument.identyfikator.typ === 'znak sprawy'
										) {
											casesSuccessObject.push(xmlContent.dokument.identyfikator.wartosc);
										}
									}
								} else {
									casesSuccessObject.push(extractedFile.extractedName);
								}
								if (
									fileDirectory[fileDirectory.length - 1] &&
									fileDirectory[fileDirectory.length - 1] !== ''
								) {
									casesToPrint.push(`sprawy/${fileDirectory[fileDirectory.length - 1]}`);
									caseFileNames.push(fileDirectory[fileDirectory.length - 1]);
								}
								if (!fileDirectory[fileDirectory.length - 1].endsWith('.xml')) {
									if (
										fileDirectory[fileDirectory.length - 1] !== undefined &&
										fileDirectory[fileDirectory.length - 1] !== ''
									) {
										allFilesInCaseFolderValid = false;
									}
								}
							} else if (fileDirectory.length >= index + 2 && fileDirectory[index + 1] !== '') {
								allFilesInCaseFolderValid = false;
							}
							break;
						default:
							otherFound = true;
							break;
					}
					fileDirectory.forEach(fileNameElement => {
						fileNameElement.split('').forEach(ch => {
							if (restricted.indexOf(ch) !== -1) {
								invalidFileNameFound = true;
								result.errors.push({
									errorMsg:
										'Nazwy plików w paczce nie sa poprawne. (nie mogą zawierać znaków: *?:=,;/\\)',
								});
							}
						});
					});
				}
			});
			metadatas.forEach(metadata => {
				const buf = Buffer.from(metadata.buffer);
				const detected = Encoding.detect(buf);
				if (detected !== 'UTF8') {
					console.log(`Encoding for ${metadata.name} is ${detected}`);
					filesWithNOUtf8Encoding.push(metadata.extractedName);
				}
			});
			if (otherFound) {
				result.success = false;
				additionalObjects = 'TAK';
				result.errors.push({
					errorMsg: 'W paczce archiwalnej oprócz głównych folderów występują także inne obiekty',
				});
			}
			if (!documentsFound || !casesFound || !metadataFound) {
				if (isFromOrderedValidation) {
					result.errors.push({
						errorMsg:
							'Struktura głównych plików (katalogów) w paczce archiwalnej jest niepoprawna.',
					});
				}
				result.success = false;
				result.errors.push({
					errorMsg:
						'Struktura głównych katalogów w paczce jest niepoprawna! Nie można zwalidować poszczególnych plików paczki archiwalnej.',
				});
				correctFolders = 'NIE';
				if (!metadataFound) {
					metadataDirPresent = false;
					result.errors.push({
						errorMsg: "Paczka nie zawiera folderu 'metadane'",
					});
				}
				if (!casesFound) {
					caseDirPresent = false;
				}
				// result.successDetails = JSON.stringify({
				// 	casesFiles: [],
				// 	documentFiles: [],
				// 	correctFolders,
				// 	correctFileNames: invalidFileNameFound ? 'NIE' : 'TAK',
				// 	documentsWithoutMetadata,
				// 	metadatasWithoutDocument,
				// 	additionalObjects,
				// 	filesWithNOUtf8Encoding,
				// 	unavailableValidation,
				// }).toString();
				// return result;
			}
			//	if (metadataFileNamesList.length === documentSuccessObject.length) {
			// zanim sprawdzimy xsd sprawdzmy kazdy plik w folderze metadane ma swoj odpowiednik w folderze dokumenty
			documentSuccessObject.forEach(d => {
				if (d !== '') {
					if (!metadataFileNamesList.includes(`${d}.xml`)) {
						documentsWithoutMetadata.push(d);
					}
				}
			});
			if (documentsWithoutMetadata.length > 0) {
				documentsWithoutMetadata.forEach(d => {
					if (d !== '') {
						result.success = false;
						result.errors.push({
							errorMsg: `Dokument ${d} nie posiada w folderze metadane odpowiednika ${d}.xml`,
						});
					}
				});
			}
			// oraz odwrotnie, czyli czy każdy plik w folderze dokumenty ma plik nazwa.xml w folderze metadane
			metadataFileNamesList.forEach(d => {
				if (!documentSuccessObject.includes(d.replace(new RegExp('.xml' + '$'), ''))) {
					metadatasWithoutDocument.push(d);
				}
			});
			if (metadatasWithoutDocument.length > 0) {
				metadatasWithoutDocument.forEach(d => {
					result.success = false;
					result.errors.push({
						errorMsg: `Metadana ${d} nie posiada w folderze dokumenty swojego odpowiednika${
							d ? ` ${d.replace('.xml', '')}` : ''
						}`,
					});
				});
			}
			// }
			if (!(metadataFileNamesList.length === documentSuccessObject.length)) {
				result.errors.push({
					errorMsg: 'Nie zgadza się liczba plików dokumentów i metadanych',
				});
				metadataAndDocumentSameCount = 'NIE';
				if (metadataFileNamesList.length > documentSuccessObject.length) {
					// do walidacji struktury
					metadatasOver = true;
				}
			}
			// 6. Czy w paczce archiwalnej w folderze 'dokumenty' znajdują się podfoldery niezawierające choćby jednego pliku? NIE MOGĄ BYĆ
			// console.log('6');
			folderNames.forEach(folderName => {
				if (folders[folderName] === 0) {
					result.errors.push({
						errorMsg: 'Paczka archiwalna zawiera puste foldery w folderze dokumenty.',
					});
					emptyFolders = 'TAK';
				}
			});
			if (!emptyFolders) {
				emptyFolders = 'NIE';
			}
			// 7. Czy w paczce archiwalnej w folderze 'dokumenty' każdy obiekt (plik lub podfolder) ma swój odpowiednik w folderze 'metadane' - tj. plik o nazwie takiej jak obiekt (nazwa obiektu wraz z ewentualnym rozszerzeniem) o rozszerzeniu 'xml'? Przykład: obiekt 'wniosek.pdf' albo 'decyzja' w folderze 'dokumenty' ma odpowiednik plik 'wniosek.pdf.xml' albo 'decyzja.xml' w folderze 'metadane'. MUSI BYĆ
			// 8. Czy w paczce archiwalnej w folderze 'metadane' każdy plik ma swój odpowiednik w folderze 'dokumenty' - tj. obiekt (plik lub podfolder) o nazwie takiej jak obiekt (tj. sama nazwa pliku bez rozszerzenia)? Przykład: plik 'wniosek.pdf.xml' albo 'decyzja.xml' w folderze 'metadane' ma odpowiednik obiekt 'wniosek.pdf' albo 'decyzja' w folderze 'dokumenty'. MUSI BYĆ
			// console.log('7,8');
			// to juz jest w weryfikcaji struktury

			// 9. Czy w paczce archiwalnej w folderze 'sprawy' występują inne obiekty niż tylko pliki i tylko o rozszerzeniu XML? NIE MOGĄ BYĆ
			// console.log('9');
			if (!allFilesInCaseFolderValid) {
				result.errors.push({
					errorMsg: 'Folder sprawy nie zawiera tylko plików o rozszerzeniu .xml',
				});
				casesNotOnlyXml = 'TAK';
			} else {
				casesNotOnlyXml = 'NIE';
			}
			// 12. Czy w paczce archiwalnej w folderze 'metadane' w każdym pliku wskazano metadaną 'grupowanie' (Metadane-1.7.xsd - podelement 'grupowanie' elementu 'dokument') i czy podelement grupowania 'kod' ma wartość, której odpowiednik (plik o nazwie równej wartości 'kod' i rozszerzeniu 'xml') znajduje się w folderze 'sprawy'? MUSI BYĆ
			// console.log('12');
			const caseFileMetadataCount = [];
			this.state.parsedFiles.forEach(parsedFile => {
				caseFileMetadataCount[parsedFile.data.extractedName] = 0;
			});

			// const metadatas = [];
			// const caseIdsFromDocumentsTag = [];
			// this.state.extractedFiles.forEach(extractedFile => {
			// 	if (!extractedFile.name.includes('@LongLink')) {
			// 		const fileDirectory = extractedFile.name.split(/[\/\\]/);
			// 		if (fileDirectory.length >= 2 && fileDirectory[fileDirectory.length - 1] !== '') {
			// 			switch (fileDirectory[0]) {
			// 				case 'metadane':
			// 					extractedFile.extractedName = fileDirectory[fileDirectory.length - 1];
			// 					extractedFile.fileType = 'Metadane';
			// 					metadatas.push(extractedFile);
			// 					break;

			// 				default:
			// 					break;
			// 			}
			// 		}
			// 	}
			// });
			let allMetadataGroupedInExistingCase = true;
			this.state.parsedFiles.forEach(parsedFile => {
				caseFileMetadataCount[parsedFile.data.extractedName] = 0;
			});

			metadatas.forEach(metadata => {
				const content = new TextDecoder().decode(new Uint8Array(metadata.buffer));
				let xmlContent;
				xml2js.parseString(content, { explicitArray: false }, (err, result) => {
					if (err !== undefined && err !== null) {
						// const errorMsg = `Plik metadanych nie jest poprawnym dokumentem XML ${metadata.extractedName}`;
						// if (
						// 	result &&
						// 	result.errors &&
						// 	result.errors.filter(e => e.errorMsg === errorMsg).length <= 0
						// ) {
						// 	result.success = false;
						// 	result.errors.push({
						// 		errorMsg,
						// 	});
						// }
					}
					xmlContent = result;
				});
				let existsGroupingCaseIdTag = true;

				if (xmlContent !== undefined && xmlContent !== null) {
					if (
						xmlContent.dokument &&
						xmlContent.dokument.grupowanie &&
						xmlContent.dokument.grupowanie instanceof Array
					) {
						xmlContent.dokument.grupowanie.forEach(g => {
							if (g.typ === 'znak sprawy') {
								existsGroupingCaseIdTag = true;
								caseIdsFromDocumentsTag.push(g.kod);
								const shouldHasCaseFileName = `${g.kod}.xml`;
								const casesWithNameOfCurrentCaseIdPlusXmlExtension = caseFileNames.filter(
									c => c === shouldHasCaseFileName
								);
								if (casesWithNameOfCurrentCaseIdPlusXmlExtension.length !== 1) {
									// pewnie nie ma takiego pliku sprawy o takiej nazwie jak kod plus .xml
									allMetadataGroupedInExistingCase = false;
								}
							}
						});
					} else {
						if (
							xmlContent &&
							xmlContent.dokument &&
							xmlContent.dokument.grupowanie &&
							xmlContent.dokument.grupowanie.typ &&
							xmlContent.dokument.grupowanie.typ === 'znak sprawy'
						) {
							existsGroupingCaseIdTag = true;
							caseIdsFromDocumentsTag.push(xmlContent.dokument.grupowanie.kod);
							const shouldHasCaseFileName = `${xmlContent.dokument.grupowanie.kod}.xml`;
							const casesWithNameOfCurrentCaseIdPlusXmlExtension = caseFileNames.filter(
								c => c === shouldHasCaseFileName
							);
							if (casesWithNameOfCurrentCaseIdPlusXmlExtension.length !== 1) {
								// pewnie nie ma takiego pliku sprawy o takiej nazwie jak kod plus .xml
								allMetadataGroupedInExistingCase = false;
							}
						} else {
							existsGroupingCaseIdTag = false;
						}
					}
				}
				if (metadata.extractedName && !existsGroupingCaseIdTag) {
					fileNamesWithNoGroupingTag.push(metadata.extractedName);
				}
				if (
					metadata.extractedName &&
					metadata.extractedName !== 'null' &&
					!allMetadataGroupedInExistingCase
				) {
					result.success = false;
					result.errors.push({
						errorMsg:
							'W folderze sprawy brakuje plików o nazwach, które występują w metadanych jako znak sprawy',
					});
				}
			});

			// 13. Czy w paczce archiwalnej w folderze 'sprawy' są jakiekolwiek pliki, których nazwa (bez rozszerzenia) nie ma odpowiednika w folderze 'metadane' w choć jednym pliku, tj. w wartości podelementu 'kod' grupowania? NIE MOŻE BYĆ
			// let allCasesHasAtLeastOneMetadata = true;
			caseFileNames.forEach(caseFileName => {
				if (caseFileName && caseFileName !== '') {
					if (!caseIdsFromDocumentsTag.includes(caseFileName.replace('.xml', ''))) {
						caseFilesWithNoMetadataAssigment.push(caseFileName);
						result.success = false;
						result.errors.push({
							errorMsg: `Plik ${caseFileName} nie ma odpowiednika w folderze 'metadane' w choć jednym pliku, tj. w wartości podelementu 'kod' grupowania`,
						});
					}
				}
			});
			let toPrepareList = [];
			console.log('parsedFiles', this.state.parsedFiles);
			console.log('parsedDocuments', this.state.parsedDocuments);
			if (this.state.parsedFiles && this.state.parsedFiles.length > 0) {
				// toPrepareList = this.state.parsedFiles;
			} else {
			}
			toPrepareList = this.state.parsedDocuments;
			if(this.state.caseParsedFiles){
				toPrepareList = toPrepareList.concat(this.state.caseParsedFiles);
			}
			const startIndex = 0;
			const limit = 50;
			const xmlList = this.prepareXmlList(toPrepareList, startIndex, limit);
			this.setState(
				{
					structureVerificationInProgress: isFromOrderedValidation
						? this.state.structureVerificationInProgress
						: true,
				},
				() => {
					this.service.validateXmls(xmlList).then(res => {
						this.validateXmlsProcess(
							resolve,
							res,
							startIndex,
							limit,
							toPrepareList,
							result,
							isFromOrderedValidation,
							casesToPrint,
							documentsFileNames,
							correctFolders,
							invalidFileNameFound,
							documentsWithoutMetadata,
							metadatasWithoutDocument,
							[],
							[],
							[],
							emptyFolders,
							casesNotOnlyXml,
							caseFilesWithNoMetadataAssigment,
							fileNamesWithNoGroupingTag,
							metadataFileNamesList,
							allMetadataGroupedInExistingCase,
							additionalObjects,
							filesWithNOUtf8Encoding,
							metadataAndDocumentSameCount,
							metadatasOver,
							metadataDirPresent,
							caseDirPresent,
							packageFileNamesInUtf8Encoding
						);
					});
				}
			);
			//	}
			//	}
		});
	}

	validateXmlsProcess(
		resolve,
		res,
		startIndex,
		limit,
		toPrepareList,
		result,
		isFromOrderedValidation,
		casesToPrint,
		documentsFileNames,
		correctFolders,
		invalidFileNameFound,
		documentsWithoutMetadata,
		metadatasWithoutDocument,
		metadatasWithMissingTags,
		casesWithMissingTags,
		incorrectMetadataXsdName,
		emptyFolders,
		casesNotOnlyXml,
		caseFilesWithNoMetadataAssigment,
		fileNamesWithNoGroupingTag,
		metadataFileNamesList,
		allMetadataGroupedInExistingCase,
		additionalObjects,
		filesWithNOUtf8Encoding,
		metadataAndDocumentSameCount,
		metadatasOver,
		metadataDirPresent,
		caseDirPresent,
		packageFileNamesInUtf8Encoding
	) {
		console.log('validateXmls', toPrepareList.length, startIndex, limit, res);
		if (res.errors !== undefined && res.errors !== null && res.errors.length > 0) {
			result.success = false;
			if (res.successDetails) {
				let successObject = {};
				try {
					successObject = JSON.parse(res.successDetails);
					incorrectMetadataXsdName = incorrectMetadataXsdName.concat(
						successObject.incorrectMetadataXsdName
					);
					casesWithMissingTags = casesWithMissingTags.concat(successObject.casesWithMissingTags);
					metadatasWithMissingTags = metadatasWithMissingTags.concat(
						successObject.metadatasWithMissingTags
					);
				} catch (e) {
					console.log('error parsing success details', e);
					successObject = undefined;
				}
			}
			res.errors.forEach(error => {
				result.errors.push({
					errorMsg: error.errorMsg,
				});
			});
		}
		startIndex = startIndex + limit;
		if (toPrepareList.length > startIndex) {
			const xmlList = this.prepareXmlList(toPrepareList, startIndex, limit);
			this.service.validateXmls(xmlList).then(res => {
				this.validateXmlsProcess(
					resolve,
					res,
					startIndex,
					limit,
					toPrepareList,
					result,
					isFromOrderedValidation,
					casesToPrint,
					documentsFileNames,
					correctFolders,
					invalidFileNameFound,
					documentsWithoutMetadata,
					metadatasWithoutDocument,
					metadatasWithMissingTags,
					casesWithMissingTags,
					incorrectMetadataXsdName,
					emptyFolders,
					casesNotOnlyXml,
					caseFilesWithNoMetadataAssigment,
					fileNamesWithNoGroupingTag,
					metadataFileNamesList,
					allMetadataGroupedInExistingCase,
					additionalObjects,
					filesWithNOUtf8Encoding,
					metadataAndDocumentSameCount,
					metadatasOver,
					metadataDirPresent,
					caseDirPresent,
					packageFileNamesInUtf8Encoding
				);
			});
		} else {
			this.setState(
				{
					structureCompareReport: isFromOrderedValidation
						? this.state.structureCompareReport
						: result,
					structureVerificationInProgress: isFromOrderedValidation
						? this.state.structureVerificationInProgress
						: false,
				},
				() => {
					const distinct = (value, index, self) => {
						return self.indexOf(value) === index;
					};
					result.successDetails = JSON.stringify({
						casesFiles: casesToPrint.sort(),
						documentFiles: documentsFileNames.sort(),
						correctFolders: correctFolders ? 'NIE' : 'TAK',
						correctFileNames: invalidFileNameFound ? 'NIE' : 'TAK',
						documentsWithoutMetadata,
						metadatasWithoutDocument,
						metadatasWithMissingTags: metadatasWithMissingTags.filter(distinct),
						casesWithMissingTags: casesWithMissingTags.filter(distinct),
						incorrectMetadataXsdName: incorrectMetadataXsdName.filter(distinct),
						emptyFolders,
						casesNotOnlyXml,
						caseFilesWithNoMetadataAssigment,
						fileNamesWithNoGroupingTag,
						metadataFileNames: metadataFileNamesList.sort(),
						documentsFileNames: documentsFileNames.sort(),
						correctCaseFileName: allMetadataGroupedInExistingCase ? 'TAK' : 'NIE',
						additionalObjects,
						filesWithNOUtf8Encoding,
						metadataAndDocumentSameCount,
						metadatasOver,
						metadataDirPresent: metadataDirPresent ? 'TAK' : 'NIE',
						caseDirPresent: caseDirPresent ? 'TAK' : 'NIE',
						packageFileNamesInUtf8Encoding: packageFileNamesInUtf8Encoding ? 'TAK' : 'NIE',
					}).toString();
					if (!isFromOrderedValidation) {
						return result;
					} else {
						return resolve(result);
					}
				}
			);
		}
	}

	prepareXmlList(toPrepareList, startIndex, limit) {
		const xmlList = [];
		if (startIndex === null || Number.isNaN(startIndex)) {
			startIndex = 0;
		}
		if (limit === null || Number.isNaN(limit)) {
			limit = 1000000;
		}
		if (startIndex + limit > toPrepareList.length) {
			limit = startIndex + limit;
		}
		for (let index = startIndex; index < startIndex + limit; index++) {
			const parsedFile = toPrepareList[index];
			if (parsedFile !== null && parsedFile !== undefined) {
				const xmlContent = new TextDecoder().decode(new Uint8Array(parsedFile.data.buffer));
				xmlList.push({
					name: parsedFile.data.extractedName,
					content: xmlContent,
					xsdInfo: this.extractXsd(xmlContent),
					type: parsedFile.data.fileType,
					pathInPackage:
						parsedFile && parsedFile.metadata && parsedFile.metadata.name
							? parsedFile.metadata.name
							: parsedFile.data &&
							  parsedFile.data.fileType &&
							  parsedFile.data.fileType === 'Metadane'
							? parsedFile.data.name
							: parsedFile.data && parsedFile.data.fileType && parsedFile.data.fileType === 'Sprawa'
							? parsedFile.data.originalFileName
							: null,
				});
				if (parsedFile.children !== undefined) {
					parsedFile.children.forEach(child => {
						const childContent = new TextDecoder().decode(new Uint8Array(child.metadata.buffer));
						xmlList.push({
							name: child.metadata.extractedName,
							content: childContent,
							xsdInfo: this.extractXsd(childContent),
							type: child.fileType,
							pathInPackage:
								child && child.metadata && child.metadata.name ? child.metadata.name : null,
						});
					});
				}
			}
		}
		// toPrepareList.forEach(parsedFile => {
		// 	const xmlContent = new TextDecoder().decode(new Uint8Array(parsedFile.data.buffer));
		// 	xmlList.push({
		// 		name: parsedFile.data.extractedName,
		// 		content: xmlContent,
		// 		xsdInfo: this.extractXsd(xmlContent),
		// 		type: parsedFile.data.fileType,
		// 		pathInPackage:
		// 			parsedFile && parsedFile.metadata && parsedFile.metadata.name
		// 				? parsedFile.metadata.name
		// 				: parsedFile.data && parsedFile.data.fileType && parsedFile.data.fileType === 'Metadane'
		// 				? parsedFile.data.name
		// 				: parsedFile.data && parsedFile.data.fileType && parsedFile.data.fileType === 'Sprawa'
		// 				? parsedFile.data.originalFileName
		// 				: null,
		// 	});
		// 	if (parsedFile.children !== undefined) {
		// 		parsedFile.children.forEach(child => {
		// 			const childContent = new TextDecoder().decode(new Uint8Array(child.metadata.buffer));
		// 			xmlList.push({
		// 				name: child.metadata.extractedName,
		// 				content: childContent,
		// 				xsdInfo: this.extractXsd(childContent),
		// 				type: child.fileType,
		// 				pathInPackage:
		// 					child && child.metadata && child.metadata.name ? child.metadata.name : null,
		// 			});
		// 		});
		// 	}
		// });
		return xmlList;
	}

	entirePackageValidation() {
		if (this.state.signatureVerificationInProgress) {
		} else {
			// 1. Czy paczka archiwalna to nieskompresowane archiwum w pożądanym formacie? MUSI BYĆ - fileChooser to ogarnia
			// 2. Czy istnieje prawidłowy (pozytywnie zweryfikowany) podpis paczki archiwalnej? MUSI BYĆ
			this.setState(
				{ signatureVerificationInProgress: true, entirePackageValidationInProgress: true },
				() =>
					verifySignature(
						this.state.packageTar,
						this.state.signature,
						this.entirePackageValidationStep1Success,
						this.entirePackageValidationStep1Error,
						false,
						false,
						'http://localhost:8088/'
					)
			);
		}
	}

	entirePackageValidationStep1Error(error) {
		this.onVerificationError(error);
		const errors = [];
		if (error === 'noExtension') {
			errors.push({
				errorMsg: 'Brak rozszerzenia Szafir SDK',
			});
		} else if (error === 'noHostApp') {
			errors.push({
				errorMsg: 'Brak aplikacji Szafir Host',
			});
		} else {
			errors.push({
				errorMsg: 'Podpis dołączony do paczki archiwalnej nie jest poprawny.',
			});
		}
		this.entirePackageValidationStep2(errors);
	}

	entirePackageValidationStep1Success(result) {
		this.onSignatureVerified(result);
		const errors = [];
		if (result.code !== 0 && result.code !== '0') {
			errors.push({
				errorMsg: 'Podpis dołączony do paczki archiwalnej nie jest poprawny',
			});
		}
		const successDetails = {
			correctSign: result.code !== 0 && result.code !== '0' ? 'NIE' : 'TAK',
		};
		this.entirePackageValidationStep2(errors, successDetails);
	}

	entirePackageValidationStep2(errors, successDetailsFromSuccess) {
		let successDetails;
		if (successDetailsFromSuccess) {
			successDetails = successDetailsFromSuccess;
		} else {
			successDetails = { correctSign: 'NIE' };
		}

		// 3. Czy w paczce archiwalnej znajdują się foldery: 'dokumenty', 'metadane', 'sprawy'? MUSZĄ BYĆ
		// 4. Czy w paczce archiwalnej poza w/w trzema folderami znajdują się jakiekolwiek inne obiekty (tj. foldery lub pliki)? NIE MOGĄ BYĆ
		// console.log('3,4');

		if (
			this.state.parsedFiles === undefined ||
			this.state.parsedFiles === null ||
			this.state.parsedFiles.length === 0 ||
			this.state.extractedFiles === undefined ||
			this.state.extractedFiles === null ||
			this.state.extractedFiles.length === 0
		) {
			errors.push({
				errorMsg: 'Struktura głównych plików (katalogów) w paczce archiwalnej jest nie poprawna.',
			});
			successDetails.correctFolders = 'NIE';
			// const result = {
			// 	validationType: {
			// 		value: 'ENTIRE_ORDERED_PACKAGE_VALIDATION',
			// 		label: 'Walidacja całkowita paczki',
			// 	},
			// 	success: false,
			// 	errors,
			// 	successDetails: JSON.stringify(successDetails).toString(),
			// };
			// this.setState({
			// 	entirePackageValidationReport: result,
			// 	entirePackageValidationInProgress: false,
			// });
		}
		if (this.state.extractedFiles && this.state.extractedFiles.length > 0) {
			successDetails.correctExtension = 'TAK';
		} else {
			successDetails.correctExtension = 'NIE';
		}

		const caseFileMetadataCount = [];
		const caseFileNames = [];
		this.state.parsedFiles.forEach(parsedFile => {
			// caseFileNames.push(parsedFile.data.extractedName);
			caseFileMetadataCount[parsedFile.data.extractedName] = 0;
		});
		// 15. Czy w paczce archiwalnej w folderze 'sprawy' są jakiekolwiek pliki, których nazwa (bez rozszerzenia) nie ma odpowiednika w spisie zd-odb (SpisZdawczo-odbiorczy.xsd), w podelemencie 'ZnakSprawy' elementu 'Sprawa'? NIE MOŻE BYĆ
		// 16. Czy w w spisie zd-odb (SpisZdawczo-odbiorczy.xsd), są jakiekolwiek egzemplarze podelementu 'ZnakSprawy' elementu 'Sprawa', które nie mają odpowiednika w nazwach plików (bez rozszerzenia) w folderze 'sprawy'? NIE MOŻE BYĆ
		const caseFileNamesFromContentList = [];
		if (
			this.state.contentListType !== undefined &&
			this.state.contentListType.value === 'ATTACH' &&
			this.state.contentList.file !== undefined
		) {
			let contentListXml;
			let invalidFileFormat = false;
			xml2js.parseString(this.state.contentList.file, { explicitArray: false }, (err, result) => {
				contentListXml = result;
				if (err !== undefined && err !== null) {
					invalidFileFormat = true;
				}
			});
			if (invalidFileFormat) {
			} else {
				if (
					contentListXml !== undefined &&
					contentListXml.SpisZdawczoOdbiorczy !== undefined &&
					contentListXml.SpisZdawczoOdbiorczy !== null &&
					contentListXml.SpisZdawczoOdbiorczy.Sprawa !== undefined &&
					contentListXml.SpisZdawczoOdbiorczy.Sprawa !== null &&
					contentListXml.SpisZdawczoOdbiorczy.Sprawa instanceof Array &&
					contentListXml.SpisZdawczoOdbiorczy.Sprawa.length > 0
				) {
					contentListXml.SpisZdawczoOdbiorczy.Sprawa.forEach(sprawa => {
						caseFileNamesFromContentList.push(sprawa.ZnakSprawy);
					});
				} else if (
					contentListXml !== undefined &&
					contentListXml.SpisZdawczoOdbiorczy !== undefined &&
					contentListXml.SpisZdawczoOdbiorczy !== null &&
					contentListXml.SpisZdawczoOdbiorczy.Sprawa !== undefined &&
					contentListXml.SpisZdawczoOdbiorczy.Sprawa !== null &&
					contentListXml.SpisZdawczoOdbiorczy.Sprawa instanceof Object
				) {
					caseFileNamesFromContentList.push(contentListXml.SpisZdawczoOdbiorczy.Sprawa.ZnakSprawy);
				}
			}
		} else if (
			this.state.contentListType !== undefined &&
			this.state.contentListType.value === 'CREATE'
		) {
			this.state.contentList.contentListFile.contentListRows.forEach(row => {
				caseFileNamesFromContentList.push(row.caseNumber);
			});
		}
		if (
			caseFileNames !== undefined &&
			caseFileNames !== null &&
			caseFileNamesFromContentList !== undefined &&
			caseFileNamesFromContentList !== null
		) {
			const allCaseNamesInContentList = caseFileNamesFromContentList.every(val =>
				caseFileNames.includes(val)
			);
			// const allCaseNamesInParsedPackage = caseFileNamesFromContentList.every(val =>
			// 	caseFileNames.includes(val)
			// );
			// if (
			// 	caseFileNames.length !== caseFileNamesFromContentList.length ||
			// 	!allCaseNamesInContentList ||
			// 	!allCaseNamesInParsedPackage
			// ) {
			// 	errors.push({
			// 		errorMsg: 'Metadane spraw nie zgadzają się ze spisem zdawczo-odbiorczym.',
			// 	});
			// }
			if (allCaseNamesInContentList) {
				successDetails.correctPackageWithContentList = 'TAK';
			} else {
				successDetails.correctPackageWithContentList = 'NIE';
			}
		}

		let structureResult;
		let handOverResult;
		let result;
		this.verifyStructure(true)
			.then(result => {
				return new Promise(() => {
					structureResult = result;
					if (
						structureResult &&
						structureResult.errors &&
						structureResult.errors instanceof Array
					) {
						structureResult.errors.forEach(error => errors.push(error));
					}
					let structureSuccessDetails;
					try {
						structureSuccessDetails = JSON.parse(structureResult.successDetails);
					} catch (ex) {
						console.log('Nie można sparsować success details z raportu struktury');
					}
					if (structureSuccessDetails) {
						successDetails.casesFiles = structureSuccessDetails.casesFiles;
						successDetails.documentFiles = structureSuccessDetails.documentFiles;
						successDetails.correctFolders = structureSuccessDetails.correctFolders;
						successDetails.correctFileNames = structureSuccessDetails.correctFileNames;
						successDetails.documentsWithoutMetadata =
							structureSuccessDetails.documentsWithoutMetadata;
						successDetails.metadatasWithoutDocument =
							structureSuccessDetails.metadatasWithoutDocument;
						successDetails.metadatasWithMissingTags =
							structureSuccessDetails.metadatasWithMissingTags;
						successDetails.casesWithMissingTags = structureSuccessDetails.casesWithMissingTags;
						successDetails.incorrectMetadataXsdName =
							structureSuccessDetails.incorrectMetadataXsdName;
						successDetails.emptyFolders = structureSuccessDetails.emptyFolders;
						successDetails.casesNotOnlyXml = structureSuccessDetails.casesNotOnlyXml;
						successDetails.caseFilesWithNoMetadataAssigment =
							structureSuccessDetails.caseFilesWithNoMetadataAssigment;
						successDetails.fileNamesWithNoGroupingTag =
							structureSuccessDetails.fileNamesWithNoGroupingTag;
						successDetails.correctCaseFileName = structureSuccessDetails.correctCaseFileName;
						successDetails.additionalObjects = structureSuccessDetails.additionalObjects;
						successDetails.filesWithNOUtf8Encoding =
							structureSuccessDetails.filesWithNOUtf8Encoding;
						successDetails.metadataAndDocumentSameCount =
							structureSuccessDetails.metadataAndDocumentSameCount;
						successDetails.metadataDirPresent = structureSuccessDetails.metadataDirPresent;
						successDetails.caseDirPresent = structureSuccessDetails.caseDirPresent;
						successDetails.packageFileNamesInUtf8Encoding =
							structureSuccessDetails.packageFileNamesInUtf8Encoding;
					}
					this.verifyContentList(true)
						.then(contentListResult => {
							handOverResult = contentListResult;
							if (
								handOverResult &&
								handOverResult.errors &&
								handOverResult.errors instanceof Array
							) {
								handOverResult.errors.forEach(error => errors.push(error));
							}
							let hoSuccessDetails;
							try {
								hoSuccessDetails = JSON.parse(handOverResult.successDetails);
							} catch (ex) {
								console.log('Nie można sparsować success details z raportu spisu');
							}
							if (hoSuccessDetails) {
								successDetails.caseIds = hoSuccessDetails.caseIds;
								successDetails.caseFilesInPackageMissingInHandoverList =
									hoSuccessDetails.caseFilesInPackageMissingInHandoverList;
								successDetails.handoverListCaseMissingInFiles =
									hoSuccessDetails.handoverListCaseMissingInFiles;
							}
							result = {
								validationType: {
									value: 'ENTIRE_ORDERED_PACKAGE_VALIDATION',
									label: 'Walidacja całkowita paczki',
								},
								success: errors.length === 0,
								errors,
								successDetails: JSON.stringify(successDetails).toString(),
							};
							this.setState({
								entirePackageValidationReport: result,
								// contentListVerificationInProgress: false,
								entirePackageValidationInProgress: false,
							});
							return result;
						})
						.catch(err => {
							console.log('zlapalem errora', err, result);
							let hoSuccessDetails;
							try {
								hoSuccessDetails = JSON.parse(err.successDetails);
							} catch (ex) {
								console.log('Nie można sparsować success details z raportu spisu');
							}
							if (hoSuccessDetails) {
								successDetails.caseIds = hoSuccessDetails.caseIds;
								successDetails.caseFilesInPackageMissingInHandoverList =
									hoSuccessDetails.caseFilesInPackageMissingInHandoverList;
								successDetails.handoverListCaseMissingInFiles =
									hoSuccessDetails.handoverListCaseMissingInFiles;
							}
							const hoErrors = err && err.errors ? err.errors.concat(result.errors) : result.errors;
							result = {
								validationType: {
									value: 'ENTIRE_ORDERED_PACKAGE_VALIDATION',
									label: 'Walidacja całkowita paczki',
								},
								success: false,
								errors: errors.concat(hoErrors),
								successDetails: JSON.stringify(successDetails).toString(),
							};
							this.setState(
								{
									entirePackageValidationReport: result,
									entirePackageValidationInProgress: false,
								}
								//	() => resolve(result)
							);
							return result;
						});
				});
			})
			.catch(() => {
				result = {
					validationType: {
						value: 'ENTIRE_ORDERED_PACKAGE_VALIDATION',
						label: 'Walidacja całkowita paczki',
					},
					success: errors.length === 0,
					errors,
					successDetails: JSON.stringify(successDetails).toString(),
				};
				this.setState({
					entirePackageValidationReport: result,
					entirePackageValidationInProgress: false,
				});
			});
		//	}
	}

	extractXsd(xmlContent) {
		const xsiSchemaIndex = xmlContent.indexOf('xsi:schemaLocation');
		const xsdNameIndex = xmlContent.indexOf(' ', xsiSchemaIndex);
		const xsdName = xmlContent.substring(xsdNameIndex + 1, xmlContent.indexOf('"', xsdNameIndex));
		const xsdVersionIndex = xsdName.indexOf('-');
		const xsdVersionIndexEnd = xsdName.indexOf('.xsd', xsdVersionIndex);
		const xsdVersion = xsdName.substring(xsdVersionIndex + 1, xsdVersionIndexEnd);
		return { xsdName, xsdVersion };
	}

	extractTar(event) {
		const data = event.target.result;
		const wordArray = typedarrays.create(data);
		const packageCheckSumHex = sha256(wordArray).toString(encHex);
		const packageCheckSumB64 = sha256(wordArray).toString(encB64);
		untar(data).then(
			extractedFiles => {
				this.setState({ extractedFiles, packageCheckSumHex, packageCheckSumB64 });
			},
			err => {
				console.log(err);
			},
			() => {
				// onProgress
			}
		);
	}
	checkParentFolderIndex(arr) {
		if (Array.isArray(arr)) {
			for (let i = 0; i < arr.length; i++) {
				if (arr[i] === 'dokumenty' || arr[i] === 'metadane' || arr[i] === 'sprawy') {
					return i;
				}
			}
		} else return 0;
	}

	parseFiles(runOnSuccess) {
		const { extractedFiles } = this.state;
		const documents = [];
		const metadatas = [];
		const cases = [];
		const caseParsedFiles = [];
		// const parsedDocuments = [];
		const parsingErrors = [];
		console.log('extractedFiles', extractedFiles);
		extractedFiles.forEach(extractedFile => {
			if (!extractedFile.name.includes('@LongLink')) {
				const fileDirectory = extractedFile.name.split(/[\/\\]/);
				if (fileDirectory.length >= 2 && fileDirectory[fileDirectory.length - 1] !== '') {
					const index = this.checkParentFolderIndex(fileDirectory);
					switch (fileDirectory[index]) {
						case 'dokumenty':
							extractedFile.extractedName = fileDirectory[fileDirectory.length - 1];
							extractedFile.fileType = 'Dokument';
							documents[fileDirectory[fileDirectory.length - 1].toUpperCase()] = extractedFile;
							break;
						case 'metadane':
							extractedFile.extractedName = fileDirectory[fileDirectory.length - 1];
							extractedFile.fileType = 'Metadane';
							metadatas.push(extractedFile);
							break;
						case 'sprawy':
							extractedFile.fileType = 'Sprawa';
							const content = new TextDecoder().decode(new Uint8Array(extractedFile.buffer));
							let xmlContent;
							xml2js.parseString(content, { explicitArray: false }, (err, result) => {
								xmlContent = result;
							});
							if (xmlContent !== undefined && xmlContent !== null) {
								const title = this.extractTitle(xmlContent);
								extractedFile.extractedName = title ? title : fileDirectory[index + 1];
								extractedFile.originalFileName = fileDirectory[index + 1];
								cases[extractedFile.extractedName] = extractedFile;
							} else {
								// const error = `Błąd w strukturze pliku ${extractedFile.name}`;
								// if (parsingErrors.filter(e => e.errorMsg === error).length <= 0) {
								// 	parsingErrors.push({
								// 		errorMsg: error,
								// 	});
								// }
							}
							break;
						default:
							// const error = `Błędna nazwa folderu: ${fileDirectory[0]}`;
							// if (parsingErrors.filter(e => e.errorMsg === error).length <= 0) {
							// 	parsingErrors.push({
							// 		errorMsg: error,
							// 	});
							// }
							break;
					}
				}
			}
		});
		metadatas.forEach(metadata => {
			const content = new TextDecoder().decode(new Uint8Array(metadata.buffer));
			let xmlContent;
			xml2js.parseString(content, { explicitArray: false }, (err, result) => {
				if (err !== undefined && err !== null) {
					// to parsowanie nie jest zgodne z XSD. jeśli naprawdę coś nie tak z xmlem to będzie to zwalidowane w serwisie
					// const errorMsg = `Plik metadanych nie jest poprawnym dokumentem XML ${metadata.extractedName}`;
					// if (parsingErrors.filter(e => e.errorMsg === errorMsg).length <= 0) {
					// 	parsingErrors.push({ errorMsg });
					// }
				}
				xmlContent = result;
			});
			let caseFile;
			let caseIdOnlyOnce = true;
			if (xmlContent !== undefined && xmlContent !== null) {
				if (xmlContent.dokument.grupowanie instanceof Array) {
					console.log('grupowanie jest listą');
					console.log(xmlContent.dokument.grupowanie.filter(g => g.typ === 'znak sprawy'), 'filtrowanie po tagu znak sprawy');
					const elementsWithCaseId = xmlContent.dokument.grupowanie.filter(g => g.typ === 'znak sprawy');
					if(elementsWithCaseId.length>1){
						console.log('znak sprawy wystąpił wiecej niż jeden raz. nie można przypisac dokumentu!');
						caseFile = null;
					}else {
						xmlContent.dokument.grupowanie.forEach(g => {
							if (g.typ === 'znak sprawy') {
								caseFile = cases[`${g.kod}`.toUpperCase()];
							}
						});
					}
					
				} else if (
					xmlContent &&
					xmlContent.dokument &&
					xmlContent.dokument.grupowanie &&
					xmlContent.dokument.grupowanie.kod
				) {
					caseFile = cases[`${xmlContent.dokument.grupowanie.kod}`.toUpperCase()];
				}
				if (caseFile !== undefined && caseFile !== null) {
					let caseFiles = caseFile.files;
					if (caseFiles === undefined) {
						caseFiles = [];
					}
					let document;
					if (xmlContent.dokument.identyfikator instanceof Array) {
						xmlContent.dokument.identyfikator.forEach(g => {
							if (g.typ === 'EZD.PUW.NazwaZalacznika') {
								document = documents[g.wartosc.toUpperCase()];
							}
						});
					} else {
						document = documents[xmlContent.dokument.identyfikator.wartosc.toUpperCase()];
					}
					if (document === undefined || document === null) {
						document =
							documents[
								metadata.extractedName.substring(0, metadata.extractedName.length - 4).toUpperCase()
							];
					}
					if (document === undefined || document === null) {
						const fileNameFromMeta = this.extractFileName(xmlContent);
						for (const docId in documents) {
							const sizeFromMeta = this.extractFileSize(xmlContent);
							if (
								metadata.extractedName.toUpperCase().includes(docId) &&
								documents[docId].size === sizeFromMeta
							) {
								document = documents[docId];
								if (fileNameFromMeta !== undefined) {
									document.extractedName = fileNameFromMeta;
								}
								break;
							}
						}
					}
					if (document !== undefined && document !== null) {
						document.metadata = metadata;
						const title = this.extractTitle(xmlContent);
						const createDate = this.extractCreateDate(xmlContent);
						const endDate = this.extractEndDate(xmlContent);
						const author = this.extractAuthor(xmlContent);
						const relationGui = this.extractRelationGui(xmlContent);
						const relationshipsDate = this.extractRelationsDate(xmlContent, createDate);
						document.fileInfo = {
							title,
							createDate,
							endDate,
							author,
							relationGui,
							relationshipsDate,
						};
						caseFiles.push(document);
						caseFile.files = caseFiles;
					} else {
						if (xmlContent.dokument.identyfikator instanceof Array) {
							xmlContent.dokument.identyfikator.forEach(g => {
								if (g.typ === 'EZD.PUW.NazwaZalacznika') {
									const error = `Załącznik nie istnieje ${g.wartosc}`;
									if (parsingErrors.filter(e => e.errorMsg === error).length <= 0) {
										parsingErrors.push({
											errorMsg: error,
										});
									}
								}
							});
						}
						//  else {
						// 	const error = `Dokument nie istnieje ${xmlContent.dokument.identyfikator.wartosc}`;
						// 	if (parsingErrors.filter(e => e.errorMsg === error).length <= 0) {
						// 		parsingErrors.push({
						// 			errorMsg: error,
						// 		});
						// 	}
						// }
					}
				}
				// 	const error = `Sprawa nie istnieje ${xmlContent.dokument.grupowanie.kod}`;
				// 	if (parsingErrors.filter(e => e.errorMsg === error).length <= 0) {
				// 		parsingErrors.push({
				// 			errorMsg: error,
				// 		});
				// 	}
				// }
			} else {
				// const error = `Plik metadanych nie jest poprawnym dokumentem XML ${metadata.extractedName}`;
				// if (parsingErrors.filter(e => e.errorMsg === error).length <= 0) {
				// 	parsingErrors.push({
				// 		errorMsg: error,
				// 	});
				// }
			}
		});
		const root = [];
		for (const caseId in cases) {
			const caseFile = cases[caseId];
			const content = new TextDecoder().decode(new Uint8Array(caseFile.buffer));
			let xmlContent;
			xml2js.parseString(content, { explicitArray: false }, (err, result) => {
				xmlContent = result;
			});
			if (xmlContent !== undefined && xmlContent !== null) {
				const title = this.extractTitleForView(xmlContent);
				const createDate = this.extractCreateDate(xmlContent);
				const endDate = this.extractEndDate(xmlContent);
				const author = this.extractAuthor(xmlContent);
				const relationGui = this.extractRelationGui(xmlContent);
				const relationshipsDate = this.extractRelationsDate(xmlContent, createDate);
				caseFile.fileInfo = {
					title,
					createDate,
					endDate,
					author,
					relationGui,
					relationshipsDate,
				};
				let children = undefined;
				if (caseFile.files !== undefined && caseFile.files !== null && caseFile.files.length > 0) {
					let documentIndex = 0;
					children = [];
					for (const fileId in caseFile.files) {
						const file = caseFile.files[fileId];
						children.push(file);
						documentIndex = documentIndex + 1;
					}
				}
				root.push({ data: caseFile, children });
				caseParsedFiles.push({data: caseFile});
			} else {
				const error = `Plik sprawy nie jest poprawnym dokumentem XML: ${caseId}`;
				if (parsingErrors.filter(e => e.errorMsg === error).length <= 0) {
					parsingErrors.push({
						errorMsg: error,
					});
				}
			}
		}

		const documentRoot = [];

		for (const doc in metadatas) {
			const docFile = metadatas[doc];
			const content = new TextDecoder().decode(new Uint8Array(docFile.buffer));
			let xmlContent;
			xml2js.parseString(content, { explicitArray: false }, (err, result) => {
				xmlContent = result;
			});
			if (xmlContent !== undefined && xmlContent !== null) {
				const title = this.extractTitleForView(xmlContent);
				const createDate = this.extractCreateDate(xmlContent);
				const endDate = this.extractEndDate(xmlContent);
				const author = this.extractAuthor(xmlContent);
				const relationGui = this.extractRelationGui(xmlContent);
				const relationshipsDate = this.extractRelationsDate(xmlContent, createDate);
				docFile.fileInfo = {
					title,
					createDate,
					endDate,
					author,
					relationGui,
					relationshipsDate,
				};
				let children = undefined;
				if (docFile.files !== undefined && docFile.files !== null && docFile.files.length > 0) {
					let documentIndex = 0;
					children = [];
					for (const fileId in docFile.files) {
						const file = docFile.files[fileId];
						children.push(file);
						documentIndex = documentIndex + 1;
					}
				}
				documentRoot.push({ data: docFile, children });
			}
			// else {
			// 	const error = `Plik metadanych nie jest poprawnym dokumentem XML: ${docFile}`;
			// 	if (parsingErrors.filter(e => e.errorMsg === error).length <= 0) {
			// 		parsingErrors.push({
			// 			errorMsg: error,
			// 		});
			// 	}
			// }
		}

		this.setState({ parsedFiles: root, parsedDocuments: documentRoot, parsingErrors, caseParsedFiles }, () =>
			runOnSuccess()
		);
	}

	extractRelationsDate(xmlContent, createDate) {
		if (
			xmlContent.dokument.relacja &&
			xmlContent.dokument.relacja.typ &&
			(xmlContent.dokument.relacja.typ.toUpperCase() === 'JESTDEKRETACJĄ' ||
				xmlContent.dokument.relacja.typ.toUpperCase() === 'MADEKRETACJĘ')
		) {
			return createDate;
		} else return undefined;
	}

	extractFileSize(xmlContent) {
		if (
			xmlContent.dokument.format !== undefined &&
			xmlContent.dokument.format.wielkosc !== undefined
		) {
			return parseInt(xmlContent.dokument.format.wielkosc._);
		} else {
			return undefined;
		}
	}

	extractFileName(xmlContent) {
		let result = undefined;
		if (xmlContent.dokument.identyfikator instanceof Array) {
			xmlContent.dokument.identyfikator.forEach(g => {
				if (g.typ === 'EZD.PUW.NazwaZalacznika') {
					result = g.wartosc;
				}
			});
		} else {
			if (
				xmlContent.dokument.identyfikator !== undefined &&
				xmlContent.dokument.identyfikator.wartosc !== undefined
			) {
				result = xmlContent.dokument.identyfikator.wartosc;
			}
		}
		return result;
	}
	extractCaseTitle(xmlContent) {
		if (
			xmlContent.dokument &&
			xmlContent.dokument.identyfikator !== undefined &&
			xmlContent.dokument.identyfikator.wartosc !== undefined &&
			xmlContent.dokument.identyfikator.wartosc instanceof Array &&
			xmlContent.dokument.identyfikator.wartosc.length > 0
		) {
			return xmlContent.dokument.identyfikator.wartosc[0]._;
		} else if (
			xmlContent.dokument &&
			xmlContent.dokument.identyfikator &&
			xmlContent.dokument.identyfikator.wartosc &&
			xmlContent.dokument.identyfikator.wartosc !== undefined &&
			typeof xmlContent.dokument.identyfikator.wartosc === 'string'
		) {
			return xmlContent.dokument.identyfikator.wartosc;
		} else if (
			xmlContent.dokument &&
			xmlContent.dokument.identyfikator !== undefined &&
			xmlContent.dokument.identyfikator.wartosc !== undefined &&
			xmlContent.dokument.identyfikator.wartosc instanceof Object
		) {
			return xmlContent.dokument.identyfikator.wartosc._;
		} else {
			return '';
		}
	}

	extractTitle(xmlContent) {
		if (
			xmlContent.dokument &&
			xmlContent.dokument.identyfikator !== undefined &&
			xmlContent.dokument.identyfikator.wartosc !== undefined &&
			xmlContent.dokument.identyfikator.wartosc instanceof Array &&
			xmlContent.dokument.identyfikator.wartosc.length > 0
		) {
			return xmlContent.dokument.identyfikator.wartosc[0]._;
		} else if (
			xmlContent.dokument &&
			xmlContent.dokument.identyfikator !== undefined &&
			xmlContent.dokument.identyfikator.wartosc !== undefined &&
			xmlContent.dokument.identyfikator.wartosc instanceof Object
		) {
			return xmlContent.dokument.identyfikator.wartosc._;
		} else if (
			xmlContent.dokument &&
			xmlContent.dokument.identyfikator &&
			xmlContent.dokument.identyfikator.wartosc &&
			xmlContent.dokument.identyfikator.wartosc !== undefined &&
			typeof xmlContent.dokument.identyfikator.wartosc === 'string'
		) {
			return xmlContent.dokument.identyfikator.wartosc;
		} else {
			return '';
		}
	}
	extractTitleForView(xmlContent) {
		if (
			xmlContent.dokument.tytul !== undefined &&
			xmlContent.dokument.tytul.oryginalny !== undefined &&
			xmlContent.dokument.tytul.oryginalny instanceof Array &&
			xmlContent.dokument.tytul.oryginalny.length > 0
		) {
			return xmlContent.dokument.tytul.oryginalny[0]._;
		} else if (
			xmlContent.dokument.tytul !== undefined &&
			xmlContent.dokument.tytul.oryginalny !== undefined &&
			xmlContent.dokument.tytul.oryginalny instanceof Object
		) {
			return xmlContent.dokument.tytul.oryginalny._;
		} else {
			return '';
		}
	}
	extractCreateDate(xmlContent) {
		if (xmlContent.dokument.data !== undefined && xmlContent.dokument.data instanceof Array) {
			for (const data of xmlContent.dokument.data) {
				if (typeof data.typ === 'string') {
					if (
						data.typ !== undefined &&
						data.typ !== null &&
						data.typ.toUpperCase() === 'STWORZONY'
					) {
						return new Date(data.czas);
					} else if (
						data.typ !== undefined &&
						data.typ !== null &&
						data.typ.toUpperCase() === 'DATY SKRAJNE'
					) {
						return new Date(data.od);
					}
				}
			}
		} else if (
			xmlContent.dokument.data !== undefined &&
			xmlContent.dokument.data instanceof Object &&
			xmlContent.dokument.data.typ !== undefined &&
			typeof xmlContent.dokument.data.typ === 'string' &&
			xmlContent.dokument.data.typ.toUpperCase() === 'STWORZONY'
		) {
			return new Date(xmlContent.dokument.data.czas);
		} else if (
			xmlContent.dokument.data !== undefined &&
			xmlContent.dokument.data instanceof Object &&
			xmlContent.dokument.data.typ !== undefined &&
			typeof xmlContent.dokument.data.typ === 'string' &&
			xmlContent.dokument.data.typ.toUpperCase() === 'DATY SKRAJNE'
		) {
			return new Date(xmlContent.dokument.data.od);
		}
	}

	extractEndDate(xmlContent) {
		if (xmlContent.dokument.data !== undefined && xmlContent.dokument.data instanceof Array) {
			for (const data of xmlContent.dokument.data) {
				if (typeof data.typ === 'string') {
					if (
						data.typ !== undefined &&
						data.typ !== null &&
						data.typ.toUpperCase() === 'STWORZONY'
					) {
						return new Date(data.czas);
					} else if (
						data.typ !== undefined &&
						data.typ !== null &&
						data.typ.toUpperCase() === 'DATY SKRAJNE'
					) {
						return new Date(data.do);
					}
				}
			}
		} else if (
			xmlContent.dokument.data !== undefined &&
			xmlContent.dokument.data instanceof Object &&
			xmlContent.dokument.data.typ !== undefined &&
			typeof xmlContent.dokument.data.typ === 'string' &&
			xmlContent.dokument.data.typ.toUpperCase() === 'DATY SKRAJNE'
		) {
			return new Date(xmlContent.dokument.data.do);
		}
	}

	extractRelationGui(xmlContent) {
		const result = [];
		if (xmlContent.dokument.relacja !== undefined && xmlContent.dokument.relacja instanceof Array) {
			for (const relacja of xmlContent.dokument.relacja) {
				result.push({
					typ: relacja.typ,
					identyfikator: {
						typ: relacja.identyfikator.typ,
						wartosc: relacja.identyfikator.wartosc,
					},
				});
			}
		} else if (
			xmlContent.dokument.relacja !== undefined &&
			xmlContent.dokument.relacja instanceof Object
		) {
			result.push({
				typ: xmlContent.dokument.relacja.typ,
				identyfikator: {
					typ: xmlContent.dokument.relacja.identyfikator.typ,
					wartosc: xmlContent.dokument.relacja.identyfikator.wartosc,
				},
			});
		}
		return result;
	}

	extractAuthor(xmlContent) {
		let tworca = undefined;
		if (xmlContent.dokument.tworca !== undefined && xmlContent.dokument.tworca instanceof Array) {
			for (const tworcaFromXml of xmlContent.dokument.tworca) {
				if (
					typeof tworcaFromXml.funkcja === 'string' &&
					(tworcaFromXml.funkcja.toUpperCase() === 'STWORZYL' ||
						tworcaFromXml.funkcja.toUpperCase() === 'STWORZYŁ' ||
						tworcaFromXml.funkcja.toUpperCase() === 'UTWORZYL' ||
						tworcaFromXml.funkcja.toUpperCase() === 'UTWORZYŁ')
				) {
					tworca = tworcaFromXml;
				}
			}
		} else if (
			xmlContent.dokument.tworca !== undefined &&
			xmlContent.dokument.tworca instanceof Object &&
			typeof xmlContent.dokument.tworca.funkcja === 'string' &&
			(xmlContent.dokument.tworca.funkcja.toUpperCase() === 'STWORZYL' ||
				xmlContent.dokument.tworca.funkcja.toUpperCase() === 'STWORZYŁ' ||
				xmlContent.dokument.tworca.funkcja.toUpperCase() === 'UTWORZYL' ||
				xmlContent.dokument.tworca.funkcja.toUpperCase() === 'UTWORZYŁ')
		) {
			tworca = xmlContent.dokument.tworca;
		}
		if (tworca !== undefined) {
			if (tworca.podmiot.osoba !== undefined && tworca.podmiot.osoba !== null) {
				return `${tworca.podmiot.osoba.imie} ${tworca.podmiot.osoba.nazwisko}`;
			} else if (tworca.podmiot.instytucja !== undefined && tworca.podmiot.instytucja !== null) {
				if (
					tworca.podmiot.instytucja.jednostka !== undefined &&
					tworca.podmiot.instytucja.jednostka !== null
				) {
					if (
						tworca.podmiot.instytucja.jednostka.pracownik !== undefined &&
						tworca.podmiot.instytucja.jednostka.pracownik !== null &&
						tworca.podmiot.instytucja.jednostka.pracownik.imie !== undefined &&
						tworca.podmiot.instytucja.jednostka.pracownik.imie !== null &&
						tworca.podmiot.instytucja.jednostka.pracownik.imie !== ''
					) {
						return `${tworca.podmiot.instytucja.jednostka.pracownik.imie} ${tworca.podmiot.instytucja.jednostka.pracownik.nazwisko}`;
					} else {
						return tworca.podmiot.instytucja.jednostka.nazwa;
					}
				} else {
					return tworca.podmiot.instytucja.nazwa;
				}
			}
		}
	}

	dateTemplate() {
		return moment(new Date()).format('DD-MM-YYYY');
	}

	nameTemplate(rowData) {
		return rowData.author ? `${rowData.author.name} ${rowData.author.surname}` : null;
	}

	sizeTemplate(rowData) {
		return filesize(rowData.size, { separator: ',' });
	}

	fileTemplate(rowData) {
		return (
			<DivContainer colClass="row">
				<DivContainer colClass="col-12">
					{rowData.name}
					<ActionLink
						key={'package-remove-button'}
						className="float-right"
						label="x"
						handleClick={e => {
							e.preventDefault();
							this.handleFileRemove(rowData);
						}}
					/>
				</DivContainer>
			</DivContainer>
		);
	}

	handleAfterFilesChange() {
		if (this.state.packageTar !== undefined && this.state.packageTar !== null) {
			if (window.FileReader) {
				const reader = new FileReader();
				reader.onloadend = this.extractTar.bind(this);
				reader.readAsArrayBuffer(this.state.packageTar);
				this.setState({ files: [this.state.packageTar] });
			} else {
				alert("Sorry, your browser doesn't support for preview");
			}
		}
	}

	handleAfterChecksumFilesChange() {
		if (this.state.checksum !== undefined && this.state.checksum !== null) {
			if (window.FileReader) {
				const reader = new FileReader();
				reader.onloadend = this.calculateUploadedCheckSum.bind(this);
				reader.readAsText(this.state.checksum);
			} else {
				alert("Sorry, your browser doesn't support for preview");
			}
		}
	}

	handleAfterSignatureFilesChange() {
		if (this.state.signature !== undefined && this.state.signature !== null) {
			if (window.FileReader) {
				const reader = new FileReader();
				reader.onloadend = this.processSignature.bind(this);
				reader.readAsText(this.state.signature);
			} else {
				alert("Sorry, your browser doesn't support for preview");
			}
		}
	}

	handleAfterContentListFilesChange() {
		if (
			this.state.contentList !== undefined &&
			this.state.contentList !== null &&
			this.state.contentList.attachment !== undefined &&
			this.state.contentList.attachment !== null
		) {
			if (window.FileReader) {
				const reader = new FileReader();
				reader.onloadend = this.processContentList.bind(this);
				reader.readAsText(this.state.contentList.attachment);
			} else {
				alert("Sorry, your browser doesn't support for preview");
			}
		}
	}

	handleAfterRemoveContentList() {
		this.setState(prevState => ({
			contentList: {
				...prevState.contentList,
				file: undefined,
			},
		}));
	}

	handleAfterRemoveChecksum() {
		this.setState({ uploadedCheckSum: undefined });
	}

	handleAfterRemoveSignature() {
		this.setState({ uploadedSignature: undefined });
	}

	calculateUploadedCheckSum(event) {
		const data = event.target.result;
		this.setState({ uploadedCheckSum: data });
	}

	processSignature(event) {
		const data = event.target.result;
		this.setState({ uploadedSignature: data });
	}

	processContentList(event) {
		const data = event.target.result;
		this.setState(prevState => ({
			contentList: {
				...prevState.contentList,
				file: data,
			},
		}));
	}

	handleFileRemove() {
		this.setState({
			packageTar: undefined,
			files: [],
			parsingErrors: [],
			parsedFiles: [],
			extractedFiles: [],
			signature: undefined,
			uploadedSignature: undefined,
			checksum: undefined,
			uploadedCheckSum: undefined,
			contentList: {
				contentListFile: { contentListRows: [] },
				attachment: undefined,
				fileName: 'spis-zo.xml',
				createDate: undefined,
				dateFrom: undefined,
				dateTo: undefined,
			},
			requestValidationTypeOptions: [
				{ value: 'SIGNATURE', label: 'Pieczęć elektroniczna/podpis' },
				{ value: 'CHECKSUM', label: 'Suma kontrolna' },
				{ value: 'CONTENT_LIST', label: 'Porównanie ze spisem zd-odb' },
				{ value: 'STRUCTURE_COMPARE', label: 'Porównanie ze strukturą i metadanymi' },
				{ value: 'ENTIRE_PACKAGE_VALIDATION', label: 'Walidacja całkowita paczki' },
			],
			contentListTypeOptions: [
				{ value: 'ATTACH', label: 'Załącz' },
				{ value: 'CREATE', label: 'Utwórz' },
			],
			requestValidationTypes: [],
			autoVerified: false,
			structureVerificationInProgress: false,
			signatureVerificationInProgress: false,
			entirePackageValidationInProgress: false,
			xsdFiles: [],
		});
	}

	createContentListRow() {
		const newElement = {
			id: this.state.contentList.contentListFile.contentListRows
				? this.state.contentList.contentListFile.contentListRows.length + 1
				: 1,
			caseNumber: '',
			classificationEntry: '',
			caseTitle: '',
			caseBeginDate: undefined,
			caseEndDate: undefined,
			documentCount: '',
			att: [],
		};
		return newElement;
	}

	onAddContentListRow() {
		const newElement = {
			id: this.state.contentList.contentListFile.contentListRows
				? this.state.contentList.contentListFile.contentListRows.length + 1
				: 1,
			caseNumber: '',
			classificationEntry: '',
			caseTitle: '',
			caseBeginDate: undefined,
			caseEndDate: undefined,
			documentCount: '',
			att: [],
		};
		this.setState(prevState => ({
			contentList: {
				...prevState.contentList,
				contentListFile: {
					contentListRows: [...this.state.contentList.contentListFile.contentListRows, newElement],
				},
			},
		}));
	}

	onEditContentListRow(rowData) {
		const modifiedList = this.state.contentList.contentListFile.contentListRows;
		modifiedList.forEach(m => {
			if (m.id === rowData.id) {
				m = rowData;
			}
		});
		this.setState(prevState => ({
			contentList: {
				...prevState.contentList,
				contentListFile: {
					contentListRows: modifiedList,
				},
			},
		}));
	}

	renderContentListAttachmentInfo(viewMode, rendered, type) {
		const contentListColumns = [
			{
				field: 'id',
				header: 'Lp.',
				editor: undefined,
				validator: 'REQUIRED',
			},
			{
				field: 'caseNumber',
				header: 'Znak sprawy',
				editor: 'TEXT',
				validator: 'REQUIRED',
			},
			{
				field: 'classificationEntry',
				header: 'Hasło klasyfikacyjne',
				editor: 'TEXT',
				validator: 'REQUIRED',
			},
			{
				field: 'caseTitle',
				header: 'Tytuł sprawy',
				editor: 'TEXT',
				validator: 'REQUIRED',
			},
			{
				field: 'caseBeginDate',
				header: 'Data wszczęcia sprawy',
				editor: 'CALENDAR',
				validator: 'REQUIRED',
			},
			{
				field: 'caseEndDate',
				header: 'Data zakończenia sprawy',
				editor: 'CALENDAR',
				validator: 'REQUIRED',
			},
			{
				field: 'documentCount',
				header: 'Liczba dokumentów',
				editor: 'TEXT',
				validator: 'REQUIRED',
			},
		];
		return (
			<DivContainer colClass="col-xl-12" rendered={rendered}>
				<DivContainer colClass="row">
					<InputFileChooseComponent
						colClass="col-md-12"
						file={this.state.contentList.attachment}
						uploadedFile={this.state.contentList.file}
						id="attachment"
						maxFileSize={1000000000}
						maxFiles={1}
						multiple={false}
						name="attachment"
						onChange={this.handleChange}
						onAfterStateChange={this.handleAfterContentListFilesChange}
						onAfterRemove={this.handleAfterRemoveContentList}
						stateField="contentList"
						rendered={type === 'ATTACH'}
						validator={this.validator}
						validators={
							this.validationChecked('CONTENT_LIST') ||
							this.validationChecked('ENTIRE_PACKAGE_VALIDATION')
								? 'required'
								: 'not_required'
						}
						label="Spis zdawczo-odbiorczy"
						showLabel={false}
						uploadMaxSizeParamName="CONTENT_LIST_ATT_MAX"
						baseRestApiUrl={this.props.baseRestApiUrl}
					>
						Upuść plik tutaj lub kliknij aby przesłać
					</InputFileChooseComponent>
					<InputFileUploadComponent
						colClass="col-xl-4 col-md-5 col-sm-12"
						id="contentList-attachment"
						name="attachment"
						label="Załączone pliki"
						file={this.state.contentList.attachment}
						showLabel
						restApiUrl={`${this.props.baseRestApiUrl}/${this.path}/transfer-request/file`}
						multiple={false}
						maxFileSize={10000000}
						onChange={this.handleChange}
						itemLabel="originalFileName"
						itemName="fileName"
						viewMode={'VIEW'}
						validator={this.validator}
						validators={'required'}
						stateField={'contentList'}
						validateViewMode
						rendered={type === 'CREATE'}
						messages={this.messages}
					/>
					<InputCalendarComponent
						colClass="col-xl-2 col-lg-2 col-md-6 col-sm-12"
						id="contentList-createDate"
						name="createDate"
						label="Data sporządzenia"
						placeholder="-"
						value={this.state.contentList.createDate}
						validator={this.validator}
						onChange={this.handleChange}
						viewMode={'EDIT'}
						validators={
							this.validationChecked('CONTENT_LIST') ||
							this.validationChecked('ENTIRE_PACKAGE_VALIDATION')
								? 'required|date'
								: 'not_required|date'
						}
						stateField={'contentList'}
						rendered={type === undefined}
						showIcon
					/>
					<InputCalendarRangeComponent
						colClass="col-xl-2 col-lg-2 col-md-6 col-sm-12"
						idFrom="contentList-dateFrom"
						idTo="contentList-dateTo"
						nameFrom="dateFrom"
						nameTo="dateTo"
						from={this.state.contentList.dateFrom}
						to={this.state.contentList.dateTo}
						label="Daty skrajne"
						labelFrom="Daty skrajne od"
						labelTo="Daty skrajne do"
						validator={this.validator}
						onChange={this.handleChange}
						viewMode={'EDIT'}
						validators={
							this.validationChecked('CONTENT_LIST') ||
							this.validationChecked('ENTIRE_PACKAGE_VALIDATION')
								? 'required|date'
								: 'not_required|date'
						}
						stateField={'contentList'}
						rendered={type === undefined}
					/>
					<EditableDataTableExt
						id="contentList-content"
						name="contentListRows"
						label=""
						colClass="col-xl-12 col-lg-12 col-md-12 col-sm-12"
						columns={contentListColumns}
						value={this.state.contentList.contentListFile.contentListRows}
						showLabel={false}
						validator={this.contentListValidator}
						onChange={this.handleChange}
						viewMode={viewMode}
						validators={'required'}
						stateField={'contentList.contentListFile'}
						rendered={type === 'CREATE'}
						createNewElement={this.createContentListRow.bind(this)}
						editable
						showAddButton
						showDeleteButton
						permanentDelete
						addButtonLabel="Dodaj kolejną pozycję"
						dataKey="uuid"
						onAfterRemoveElement={() => {
							const contentListRows = this.state.contentList.contentListFile.contentListRows;
							contentListRows.forEach((el, i) => {
								el.id = i + 1;
							});
							this.setState(prevState => ({
								contentList: {
									...prevState.contentList,
									contentListFile: {
										...prevState.contentList.contentListFile,
										contentListRows,
									},
								},
							}));
						}}
					/>
					<DivContainer
						colClass="col-md-12"
						rendered={
							viewMode !== 'VIEW' &&
							this.state.contentListType !== undefined &&
							this.state.contentListType.value === 'CREATE'
						}
					>
						<ActionButton
							label="Aktualizuj dokument"
							className="float-right"
							handleClick={e => {
								e.preventDefault();
								if (this.contentListValidator.allValid()) {
									switch (type) {
										case 'ATTACH':
											break;
										case 'CREATE':
											this.service.uploadAndCreateFile(this.state.contentList).then(result => {
												this.setState(prevState => ({
													contentList: {
														...prevState.contentList,
														attachment: result,
													},
												}));
											});
											break;
									}
								} else {
									this.contentListValidator.showMessages();
									// rerender to show messages for the first time
									this.forceUpdate();
								}
							}}
						/>
					</DivContainer>
				</DivContainer>
			</DivContainer>
		);
	}

	renderReport(selectedReports, report) {
		let result = false;
		selectedReports.forEach(sr => {
			switch (sr.value) {
				case 'SIGNATURE':
					result |= report === 'signatureReport';
					break;
				case 'CHECKSUM':
					result |= report === 'checksumReport';
					break;
				case 'CONTENT_LIST':
					result |= report === 'contentListReport';
					break;
				case 'STRUCTURE_COMPARE':
					result |= report === 'structureCompareReport';
					break;
				case 'ENTIRE_PACKAGE_VALIDATION':
					result |= report === 'entirePackageValidationReport';
					break;
			}
		});
		return result;
	}

	renderValidationReport(report) {
		let label = '';
		let inProgress = false;
		const rendered = this.renderReport(this.state.requestValidationTypes, report);
		switch (report) {
			case 'signatureReport':
				label = 'Pieczęć elektroniczna/podpis';
				inProgress = this.state.signatureVerificationInProgress;
				break;
			case 'checksumReport':
				label = 'Suma kontrolna';
				break;
			case 'contentListReport':
				label = 'Porównanie ze spisem zd-odb';
				inProgress = this.state.contentListVerificationInProgress;
				break;
			case 'structureCompareReport':
				label = 'Porównanie ze strukturą i metadanymi';
				inProgress = this.state.structureVerificationInProgress;
				break;
			case 'entirePackageValidationReport':
				label = 'Walidacja całkowita paczki';
				inProgress = this.state.entirePackageValidationInProgress;
				break;
		}
		if (rendered) {
			return (
				<DivContainer colClass="col-md-12" rendered={this.state[report]}>
					<DivContainer colClass="row">
						<InputCheckboxComponent
							colClass="col-xl-3 col-lg-3 col-md-4 col-sm-12"
							id={`${report}-success`}
							name="success"
							label={label}
							labelOnRight
							value={this.state[report] ? this.state[report].success : false}
							validator={this.validator}
							validators="not_required"
							onChange={null}
							viewMode={'EDIT'}
						/>
						<InputDropdownComponent
							id={`${report}-formatType`}
							name={`${report}-formatType`}
							label="Format"
							colClass="col-xl-2 col-lg-2 col-md-2 col-sm-2"
							value={this.state[report] ? this.state[report].formatType : null}
							optionLabel="label"
							dataKey="value"
							options={this.state.reportFormatTypeOptions}
							validator={
								report === 'signatureReport'
									? this.signatureValidationReportFormatTypeValidator
									: report === 'checksumReport'
									? this.checksumValidationReportFormatTypeValidator
									: report === 'contentListReport'
									? this.handoverListValidationReportFormatTypeValidator
									: report === 'structureCompareReport'
									? this.schemaValidationReportFormatTypeValidator
									: report === 'entirePackageValidationReport'
									? this.entirePackageValidationReportFormatTypeValidator
									: this.formatTypeValidator
							}
							onChange={(a, b, e) => this.onSelectedChange(report, e.value)}
							showClear
							viewMode={'EDIT'}
							showLabel={false}
							placeholder={'Wybierz format'}
							stateField={`${report}`}
							rendered={!inProgress}
						/>

						<ActionButton
							label="Pobierz raport"
							icon="pi pi-download"
							handleClick={this.handleGenerateAutoVerificationReport.bind(this, report)}
							rendered={!inProgress}
						/>
						{inProgress ? <span>W trakcie weryfikacji</span> : null}
					</DivContainer>
				</DivContainer>
			);
		} else {
			return null;
		}
	}

	onSelectedChange(report, v) {
		const modifiedList = JSON.parse(JSON.stringify(this.state[report]));
		if (modifiedList !== undefined) {
			modifiedList.formatType = v;
			this.setState({
				[report]: modifiedList,
			});
		}
	}

	generateReport() {}

	generateXmlReport() {}

	handleGenerateAutoVerificationReport(reportName, e) {
		e.preventDefault();
		const validator =
			reportName === 'signatureValidationReport'
				? this.signatureValidationReportFormatTypeValidator
				: reportName === 'checksumValidationReport'
				? this.checksumValidationReportFormatTypeValidator
				: reportName === 'contentListReport'
				? this.handoverListValidationReportFormatTypeValidator
				: reportName === 'schemaValidationReport'
				? this.schemaValidationReportFormatTypeValidator
				: reportName === 'entirePackageValidationReport'
				? this.entirePackageValidationReportFormatTypeValidator
				: this.formatTypeValidator;
		const report = this.state[reportName];
		const validations = [];
		validations.push(report);
		const reportData = {
			type: 'package',
			verificationDate: new Date(),
			validations,
		};

		const dateFrom =
			this.state.contentList && this.state.contentList.dateFrom
				? this.state.contentList.dateFrom
				: undefined;
		const dateTo =
			this.state.contentList && this.state.contentList.dateTo
				? this.state.contentList.dateTo
				: undefined;
		// to do skad to wziac?
		const materialsNumber = '';
		const positionCount = '';
		const checksumFileName =
			this.state.checksum && this.state.checksum.name ? this.state.checksum.name : '';
		const packageFileName =
			this.state.packageTar && this.state.packageTar.name ? this.state.packageTar.name : '';
		const signatureFileName =
			this.state.signature && this.state.signature.name ? this.state.signature.name : '';
		const additionalData = {
			dateFrom,
			dateTo,
			materialsNumber,
			positionCount,
			checksumFileName,
			packageFileName,
			signatureFileName,
		};
		if (validator.allValid()) {
			if (report && report.formatType && report.formatType.value === 'PDF') {
				this.generateReport(report, reportData, additionalData);
			} else if (report && report.formatType && report.formatType.value === 'XML') {
				this.generateXmlReport(report, reportData);
			}
		} else {
			validator.showMessages();
			this.forceUpdate();
		}
	}
	autoVerification(e) {
		e.preventDefault();
		if (this.validator.allValid()) {
			this.setState({ autoVerified: true });
			if (this.state.parsedFiles.length === 0) {
				this.parseFiles(this.runVerification);
			} else {
				this.runVerification();
			}
		} else {
			this.validator.showMessages();
			// rerender to show messages for the first time
			this.forceUpdate();
		}
	}

	runVerification() {
		let runEntirePackageValidation = false;
		this.state.requestValidationTypes.forEach(validationType => {
			runEntirePackageValidation |= validationType.value === 'ENTIRE_PACKAGE_VALIDATION';
		});
		this.state.requestValidationTypes.forEach(validationType => {
			if (validationType.value === 'SIGNATURE') {
				if (!runEntirePackageValidation) {
					this.verifySignatureTask();
				}
			} else if (validationType.value === 'CHECKSUM') {
				this.setState({ checksumReport: this.verifyChecksum() });
			} else if (validationType.value === 'CONTENT_LIST') {
				this.verifyContentList()
					.then(contentListReport =>
						this.setState({ contentListReport, contentListVerificationInProgress: false })
					)
					.catch(err => {
						const result = {
							validationType: {
								value: 'HANDOVER_LIST_VALIDATION',
								label: 'Weryfikacja spisu zdawczo-odbiorczego',
							},
							success: false,
							errors: err.errors,
						};
						this.setState({
							contentListReport: result,
							contentListVerificationInProgress: false,
						});
					});
			} else if (validationType.value === 'STRUCTURE_COMPARE') {
				this.verifyStructure();
			} else if (validationType.value === 'ENTIRE_PACKAGE_VALIDATION') {
				this.entirePackageValidation();
			}
		});
	}

	prepareHeaderItems() {
		return [
			{
				label: 'Wczytaj paczkę archiwalną',
				type: 'LABEL',
				className: '',
			},
		];
	}

	render() {
		const columns = [
			{ header: 'Nazwa pliku', body: this.fileTemplate },
			{ header: 'Data wgrania', body: this.dateTemplate },
			// {field: 'controlSum', header: 'Suma kontrolna'},
			{ header: 'Autor', body: this.nameTemplate },
			{ header: 'Rozmiar paczki', body: this.sizeTemplate },
		];
		const dynamicColumns = columns.map(col => {
			return (
				<Column
					columnKey={col.key ? col.key : col.field ? col.field : col.header}
					key={col.key ? col.key : col.field ? col.field : col.header}
					field={col.field}
					header={col.header}
					body={col.body}
					className={col.className}
					sortable={col.sortable}
				/>
			);
		});

		return (
			<div>
				<CustomMessages ref={el => (this.messages = el)} />
				<NoSzafirExtDialog
					visible={this.state.visibleNoSzafirExtDialog}
					onClose={() => this.setState({ visibleNoSzafirExtDialog: false })}
				/>
				<NoSzafirHostDialog
					visible={this.state.visibleNoSzafirHostDialog}
					onClose={() => this.setState({ visibleNoSzafirHostDialog: false })}
				/>
				<Card header={this.renderHeader()}>
					<DivContainer colClass="row">
						<CustomLabel label="Paczka archiwalna" colClass="col-md-12" />
						{this.state.files === undefined || this.state.files.length === 0 ? (
							<InputFileChooseComponent
								accepts={['.tar']}
								colClass="col-md-12"
								label="Paczka archiwalna"
								showLabel={false}
								file={this.state.packageTar}
								id="package-file"
								maxFileSize={1000000000}
								maxFiles={1}
								multiple={false}
								name="packageTar"
								onChange={this.handleChange}
								onAfterStateChange={this.handleAfterFilesChange}
								onAfterRemove={this.handleFileRemove}
								stateField=""
								uploadMaxSizeParamName="PACK_ATT_MAX"
								baseRestApiUrl={this.props.baseRestApiUrl}
							>
								Upuść plik tutaj lub kliknij aby przesłać
							</InputFileChooseComponent>
						) : (
							<DivContainer colClass="col-xl-12">
								<DivContainer colClass="row">
									<DivContainer colClass="col-xl-12 form-group">
										{/* TODO label */}
										<CustomDataTable
											key="data-table"
											emptyMessage="Brak paczki"
											value={this.state.files}
											totalRecords={1}
											responsive
										>
											{dynamicColumns}
										</CustomDataTable>
									</DivContainer>
								</DivContainer>
							</DivContainer>
						)}
						<CustomLabel
							label="Suma kontrolna"
							colClass="col-md-12"
							rendered={this.state.files !== undefined && this.state.files.length > 0}
						/>
						<InputFileChooseComponent
							colClass="col-md-12"
							file={this.state.checksum}
							uploadedFile={this.state.uploadedCheckSum}
							id="checksum"
							maxFileSize={1000000000}
							maxFiles={1}
							multiple={false}
							name="checksum"
							onChange={this.handleChange}
							onAfterRemove={this.handleAfterRemoveChecksum}
							onAfterStateChange={this.handleAfterChecksumFilesChange}
							stateField=""
							validator={this.validator}
							validators={
								this.validationChecked('CHECKSUM') ||
								this.validationChecked('ENTIRE_PACKAGE_VALIDATION')
									? 'required'
									: 'not_required'
							}
							label="Suma kontrolna"
							showLabel={false}
							rendered={this.state.files !== undefined && this.state.files.length > 0}
							uploadMaxSizeParamName="SUM_ATT_MAX"
							baseRestApiUrl={this.props.baseRestApiUrl}
						>
							Upuść plik tutaj lub kliknij aby przesłać
						</InputFileChooseComponent>
						<CustomLabel
							label="Weryfikacja podpisu"
							colClass="col-md-12"
							rendered={this.state.files !== undefined && this.state.files.length > 0}
						/>
						<InputFileChooseComponent
							colClass="col-md-12"
							file={this.state.signature}
							uploadedFile={this.state.uploadedSignature}
							id="signature"
							maxFileSize={1000000000}
							maxFiles={1}
							multiple={false}
							name="signature"
							onChange={this.handleChange}
							onAfterRemove={this.handleAfterRemoveSignature}
							onAfterStateChange={this.handleAfterSignatureFilesChange}
							stateField=""
							validator={this.validator}
							validators={
								this.validationChecked('SIGNATURE') ||
								this.validationChecked('ENTIRE_PACKAGE_VALIDATION')
									? 'required'
									: 'not_required'
							}
							label="Weryfikacja podpisu"
							showLabel={false}
							rendered={this.state.files !== undefined && this.state.files.length > 0}
							uploadMaxSizeParamName="SIG_ATT_MAX"
							baseRestApiUrl={this.props.baseRestApiUrl}
						>
							Upuść plik tutaj lub kliknij aby przesłać
						</InputFileChooseComponent>
						<CustomLabel
							label="Spis zdawczo-odbiorczy"
							colClass="col-md-12"
							rendered={this.state.files !== undefined && this.state.files.length > 0}
						/>
						<InputSelectButtonComponent
							colClass="col-xl-12 col-lg-12 col-md-12 col-sm-12"
							id="contentListType"
							name="contentListType"
							value={this.state.contentListType}
							options={this.state.contentListTypeOptions}
							optionLabel="label"
							label="Spis zdawczo-odbiorczy"
							validator={this.validator}
							validators={
								this.validationChecked('CONTENT_LIST') ||
								this.validationChecked('ENTIRE_PACKAGE_VALIDATION')
									? 'required'
									: 'not_required'
							}
							onChange={this.handleChange}
							onAfterStateChange={() => {
								this.setState(prevState => ({
									contentList: {
										...prevState.contentList,
										file: undefined,
										attachment: undefined,
									},
								}));
								this.handleAfterRemoveContentList();
								if (this.state.contentListType && this.state.contentListType.value === 'ATTACH') {
									delete this.validator.fields['contentList-attachment'];
								}
								if (this.state.contentListType && this.state.contentListType.value === 'ATTACH') {
									delete this.validator.fields['contentList-attachment'];
								}
							}}
							viewMode={'EDIT'}
							showLabel={false}
							stateField=""
							rendered={this.state.files !== undefined && this.state.files.length > 0}
						/>
						{this.renderContentListAttachmentInfo(
							'VIEW',
							this.state.files !== undefined && this.state.files.length > 0
						)}
						<CustomLabel
							label="Załącz spis zdawczo-odbiorczy"
							colClass="col-md-12"
							rendered={
								this.state.contentListType !== undefined &&
								this.state.contentListType.value === 'ATTACH'
							}
						/>
						{this.renderContentListAttachmentInfo(
							'EDIT',
							this.state.files !== undefined &&
								this.state.files.length > 0 &&
								this.state.contentListType !== undefined &&
								this.state.contentListType.value === 'ATTACH',
							'ATTACH'
						)}
						<CustomLabel
							label="Utwórz spis zdawczo-odbiorczy"
							colClass="col-md-12"
							rendered={
								this.state.contentListType !== undefined &&
								this.state.contentListType.value === 'CREATE'
							}
						/>
						{this.renderContentListAttachmentInfo(
							'EDIT',
							this.state.files !== undefined &&
								this.state.files.length > 0 &&
								this.state.contentListType !== undefined &&
								this.state.contentListType.value === 'CREATE',
							'CREATE'
						)}
						<CustomLabel
							label="Automatyczna weryfikacja"
							colClass="col-md-12"
							rendered={this.state.files !== undefined && this.state.files.length > 0}
						/>
						<InputMultiCheckboxComponent
							colClass="col-xl-12 col-lg-12 col-md-12 col-sm-12"
							itemColClass="col-xl-12 col-lg-12 col-md-12 col-sm-12"
							id="requestValidationTypes"
							name="requestValidationTypes"
							value={this.state.requestValidationTypes}
							options={this.state.requestValidationTypeOptions}
							valueLabel="label"
							valueKey="value"
							label="Zakres weryfikacji"
							labelOnRight
							validator={this.validator}
							validators={'array_required'}
							onChange={this.handleChange}
							viewMode={'EDIT'}
							showLabel
							stateField=""
							rendered={this.state.files !== undefined && this.state.files.length > 0}
							disabled={this.state.autoVerified}
						/>
						<ActionButton
							colClass="col-md-12"
							handleClick={this.autoVerification}
							label="Rozpocznij weryfikację"
							rendered={
								this.state.files !== undefined &&
								this.state.files.length > 0 &&
								!this.state.autoVerified
							}
						/>
						<DivContainer
							colClass="col-md-12"
							rendered={
								this.state.files !== undefined &&
								this.state.files.length > 0 &&
								this.state.autoVerified
							}
						>
							<DivContainer colClass="row">
								<CustomLabel label="Wynik" className="p-label" colClass="col-md-12" />
								{this.renderValidationReport('signatureReport')}
								{this.renderValidationReport('checksumReport')}
								{this.renderValidationReport('contentListReport')}
								{this.renderValidationReport('structureCompareReport')}
								{this.renderValidationReport('entirePackageValidationReport')}
								<ActionButton
									handleClick={e => {
										e.preventDefault();
										this.setState({
											autoVerified: false,
											signatureReport: undefined,
											checksumReport: undefined,
											contentListReport: undefined,
											structureCompareReport: undefined,
											entirePackageValidationReport: undefined,
										});
									}}
									label="Weryfikuj ponownie"
								/>
							</DivContainer>
						</DivContainer>
					</DivContainer>
				</Card>
			</div>
		);
	}
}

BaseValidationStandaloneContainer.defaultProps = {
	baseRestApiUrl: 'http://localhost:8088/o/rest-api',
};

BaseValidationStandaloneContainer.propTypes = {
	baseRestApiUrl: PropTypes.string.isRequired,
};

export default BaseValidationStandaloneContainer;
