import React from 'react';
import PropTypes from 'prop-types';
import { TreeTable } from 'primereact/treetable';
import { Column } from 'primereact/column';
import { Card } from 'primereact/card';
import untar from 'js-untar';
import moment from 'moment';
import xml2js from 'xml2js';
import { TextDecoder } from 'text-encoding';
import CustomMessages from '../components/CustomMessages';
import BaseContainer from './BaseContainer';
import CriteriaTextComponent from '../components/criteria/CriteriaTextComponent';
import CriteriaCalendarRangeComponent from '../components/criteria/CriteriaCalendarRangeComponent';
import ActionButton from '../components/ActionButton';
import ActionLink from '../components/ActionLink';
import DivContainer from '../components/DivContainer';
import DownloadContent from '../components/DownloadContent';
import CustomLabel from '../components/CustomLabel';
import InputFileChooseComponent from '../components/inputs/InputFileChooseComponent';

class BaseVisualisationStandaloneContainer extends BaseContainer {
	constructor(props) {
		super(props, undefined);
		this.state = {
			loading: true,
			extractedFiles: [],
			expandedKeys: [],
			treeTableSource: [],
			treeTableSourceFiltered: [],
			criteria: {
				searchDocumentName: undefined,
				documentDateFrom: undefined,
				documentDateTo: undefined,
			},
		};
		this.onBeforeSent = this.onBeforeSent.bind(this);
		this.parseFiles = this.parseFiles.bind(this);
		this.actionTemplate = this.actionTemplate.bind(this);
		this.documentTemplate = this.documentTemplate.bind(this);
		this.caseTemplate = this.caseTemplate.bind(this);
		this.readTreeTableSourceFromStorage = this.readTreeTableSourceFromStorage.bind(this);
		this.handleChangeSc = this.handleChangeSc.bind(this);
		this.handleSearchFormSubmit = this.handleSearchFormSubmit.bind(this);
		this.processChildren = this.processChildren.bind(this);
		this.handleAfterFilesChange = this.handleAfterFilesChange.bind(this);
		this.handleFileRemove = this.handleFileRemove.bind(this);
		this.renderXmlDialog = this.renderXmlDialog.bind(this);
	}

	componentDidMount() {
		super.componentDidMount();
		if (!window.TextDecoder) {
			window.TextDecoder = TextDecoder;
		}
	}

	handleChangeSc(event, optionValue) {
		const varName = event.target.name;
		let varValue = event.checked !== undefined ? event.checked : event.target.value;
		if (varValue === '' || varValue === null) {
			varValue = undefined;
		}
		if (optionValue !== null && optionValue !== undefined) {
			let varValueEnum = undefined;
			if (varValue) {
				varValueEnum = varValue[optionValue];
			}
			this.setState(prevState => ({
				criteria: {
					...prevState.criteria,
					[`${varName}Obj`]: varValue,
					[varName]: varValueEnum,
				},
			}));
		} else {
			this.setState(prevState => ({
				criteria: {
					...prevState.criteria,
					[varName]: varValue,
				},
			}));
		}
	}

	addSearchArea() {
		return (
			<form
				className="form-search-criteria"
				onSubmit={e => {
					this.handleSearchFormSubmit(e);
				}}
				onKeyDown={e => {
					if (e.key === 'Enter') {
						this.handleSearchFormSubmit();
					}
				}}
				//avoid setting red border just after the page has loaded (red border because of undefined required value in FF)
				noValidate
			>
				<DivContainer colClass="col-12">
					<DivContainer colClass="row">
						<DivContainer colClass="col-criteria">
							<DivContainer colClass="row">
								<CriteriaTextComponent
									id="searchDocumentName"
									name="searchDocumentName"
									label="Nazwa dokumentu"
									colClass="col-xl-2 col-lg-2 col-md-6 col-sm-12"
									value={this.state.criteria.searchDocumentName}
									onChange={this.handleChangeSc}
									validators={this.validators}
								/>
								<CriteriaCalendarRangeComponent
									colClass="col-xl-3 col-md-6 col-sm-12"
									idFrom="documentDateFrom"
									idTo="documentDateTo"
									nameFrom="documentDateFrom"
									nameTo="documentDateTo"
									from={this.state.criteria.documentDateFrom}
									to={this.state.criteria.documentDateTo}
									label="Daty utworzenia"
									onChange={this.handleChangeSc}
									validators={this.validators}
								/>
							</DivContainer>
						</DivContainer>
						<DivContainer colClass="col-filter">
							<ActionButton
								className="float-right float-bottom"
								label="Filtruj"
								handleClick={e => {
									this.handleSearchFormSubmit(e);
								}}
							/>
						</DivContainer>
					</DivContainer>
				</DivContainer>
			</form>
		);
	}

	handleSearchFormSubmit(event) {
		if (event !== undefined) {
			event.preventDefault();
		}
		if (this.validator.allValid()) {
			const treeTableSource = this.state.treeTableSource;
			const treeTableSourceFiltered = [];
			treeTableSource.forEach(el => {
				const children = this.processChildren(el);
				let found = children !== undefined && children.length > 0;
				let foundDate = children !== undefined && children.length > 0;
				if (
					this.state.criteria.searchDocumentName === undefined ||
					this.state.criteria.searchDocumentName === '' ||
					(this.state.criteria.searchDocumentName !== undefined &&
						el.data.extractedName
							.toUpperCase()
							.includes(this.state.criteria.searchDocumentName.toUpperCase()))
				) {
					found = true;
				}
				if (
					(this.state.criteria.documentDateFrom === undefined ||
						this.state.criteria.documentDateFrom === '' ||
						(this.state.criteria.documentDateFrom !== undefined &&
							el.data &&
							el.data.fileInfo &&
							el.data.fileInfo.createDate &&
							moment(this.state.criteria.documentDateFrom).isSameOrBefore(
								moment(el.data.fileInfo.createDate)
							))) &&
					(this.state.criteria.documentDateTo === undefined ||
						this.state.criteria.documentDateTo === '' ||
						(this.state.criteria.documentDateTo !== undefined &&
							el.data &&
							el.data.fileInfo &&
							el.data.fileInfo.createDate &&
							moment(this.state.criteria.documentDateTo).isSameOrAfter(
								moment(el.data.fileInfo.createDate)
							)))
				) {
					foundDate = true;
				}
				if (
					(this.state.criteria.documentDateFrom !== undefined &&
						el.data &&
						el.data.fileInfo &&
						el.data.fileInfo.createDate &&
						moment(this.state.criteria.documentDateFrom).isAfter(
							moment(el.data.fileInfo.createDate)
						)) ||
					(this.state.criteria.documentDateTo !== undefined &&
						el.data &&
						el.data.fileInfo &&
						el.data.fileInfo.createDate &&
						moment(this.state.criteria.documentDateTo).isBefore(
							moment(el.data.fileInfo.createDate)
						))
				) {
					foundDate = false;
				}
				if (found && foundDate) {
					treeTableSourceFiltered.push({ key: el.key, data: el.data, children });
				}
			});
			this.setState({ treeTableSourceFiltered });
		} else {
			this.validator.showMessages();
			// rerender to show messages for the first time
			this.scrollToError = true;
			this.forceUpdate();
		}
	}

	processChildren(element) {
		if (
			element.children !== undefined &&
			element.children !== null &&
			element.children.length > 0
		) {
			const result = [];
			element.children.forEach(el => {
				const children = this.processChildren(el);

				let found = children !== undefined && children.length > 0;
				let foundDate = children !== undefined && children.length > 0;

				if (
					this.state.criteria.searchDocumentName === undefined ||
					this.state.criteria.searchDocumentName === '' ||
					(this.state.criteria.searchDocumentName !== undefined &&
						el.data.extractedName
							.toUpperCase()
							.includes(this.state.criteria.searchDocumentName.toUpperCase()))
				) {
					found = true;
				}
				if (
					(this.state.criteria.documentDateFrom === undefined ||
						this.state.criteria.documentDateFrom === '' ||
						(this.state.criteria.documentDateFrom !== undefined &&
							el.data &&
							el.data.fileInfo &&
							el.data.fileInfo.createDate &&
							moment(this.state.criteria.documentDateFrom).isSameOrBefore(
								moment(el.data.fileInfo.createDate)
							))) &&
					(this.state.criteria.documentDateTo === undefined ||
						this.state.criteria.documentDateTo === '' ||
						(this.state.criteria.documentDateTo !== undefined &&
							el.data &&
							el.data.fileInfo &&
							el.data.fileInfo.createDate &&
							moment(this.state.criteria.documentDateTo).isSameOrAfter(
								moment(el.data.fileInfo.createDate)
							)))
				) {
					foundDate = true;
				}
				if (
					(this.state.criteria.documentDateFrom !== undefined &&
						el.data &&
						el.data.fileInfo &&
						el.data.fileInfo.createDate &&
						moment(this.state.criteria.documentDateFrom).isAfter(
							moment(el.data.fileInfo.createDate)
						)) ||
					(this.state.criteria.documentDateTo !== undefined &&
						el.data &&
						el.data.fileInfo &&
						el.data.fileInfo.createDate &&
						moment(this.state.criteria.documentDateTo).isBefore(
							moment(el.data.fileInfo.createDate)
						))
				) {
					foundDate = false;
				}
				if (found && foundDate) {
					result.push({ key: el.key, data: el.data, children });
				}
			});
			return result;
		} else {
			return undefined;
		}
	}

	actionTemplate(node) {
		return (node.data && node.data.metadata) || node.data.fileInfo ? (
			<div>
				<div className="row">
					<div className="col-12">
						<ActionLink
							label={'Metadane'}
							className="p-link"
							variant="accent"
							handleClick={e => {
								e.preventDefault();
								this.saveCookie(
									'visualisation-standalone-xml',
									JSON.stringify(
										new TextDecoder().decode(
											new Uint8Array(
												node.data.metadata ? node.data.metadata.buffer : node.data.buffer
											)
										)
									)
								);
								this.setState({ xmlEditorVisible: true });
							}}
							key={`${node.data.key}-edit-metadata-button`}
						/>
					</div>
				</div>
			</div>
		) : null;
	}

	caseTemplate(node) {
		if (node.data.type === 'Sprawa') {
			let label;
			let fileName;
			if (node.data && node.data.fileInfo && node.data.fileInfo.title) {
				label = node.data.fileInfo.title;
				fileName = node.data.originalFileName;
			} else if (node.data) {
				label = node.data.extractedName;
				fileName = node.data.originalFileName;
			} else label = undefined;
			return label ? (
				<DownloadContent
					label={`${label} (${fileName})`}
					key={`${node.data.key}-${name}-case-download-button`}
					fileName={fileName}
					mode="LINK"
					content={node.data.buffer}
				/>
			) : (
				''
			);
		} else {
			return '';
		}
	}

	documentTemplate(node) {
		if (node.data.type === 'Sprawa') {
			return node.children !== undefined ? node.children.length : 0;
		} else if (node.data.type === 'Dokument') {
			let label;
			let fileName;
			if (node.data && node.data.fileInfo && node.data.fileInfo.title) {
				label = node.data.fileInfo.title;
				fileName = node.data.extractedName;
			} else if (node.data) {
				label = node.data.extractedName;
				fileName = node.data.extractedName;
			} else label = undefined;
			return (
				<DownloadContent
					label={`${label} (${fileName})`}
					key={`${node.data.key}-${name}-document-download-button`}
					fileName={fileName}
					mode="LINK"
					content={node.data.buffer}
				/>
			);
		} else {
			return '';
		}
	}

	attachmentTemplate(node) {
		if (node.data.type === 'Sprawa') {
			return '';
		} else if (node.data.type === 'Dokument') {
			return node.children !== undefined ? node.children.length : 0;
		} else {
			let label;
			let fileName;
			if (node.data && node.data.fileInfo && node.data.fileInfo.title) {
				label = node.data.fileInfo.title;
				fileName = node.data.extractedName;
			} else if (node.data) {
				label = node.data.extractedName;
				fileName = node.data.extractedName;
			} else label = undefined;
			return (
				<DownloadContent
					label={`${label} (${fileName})`}
					key={`${node.data.key}-${name}-attachment-download-button`}
					fileName={fileName}
					mode="LINK"
					content={node.data.buffer}
				/>
			);
		}
	}

	authorTemplate(node) {
		if (
			node.data !== undefined &&
			node.data.fileInfo !== undefined &&
			node.data.fileInfo.author !== undefined
		) {
			return node.data.fileInfo.author;
		} else {
			return '';
		}
	}

	relationTemplate(node) {
		if (
			node.data !== undefined &&
			node.data.fileInfo !== undefined &&
			node.data.fileInfo.relationGui !== undefined &&
			node.data.fileInfo.relationGui.length > 0
		) {
			let result = '';
			node.data.fileInfo.relationGui.forEach(el => {
				result = `${result}${el.typ} ${el.identyfikator.wartosc}\n`;
			});
			return result;
		} else {
			return '';
		}
	}

	createDateTemplate(format, node) {
		if (
			node.data !== undefined &&
			node.data.fileInfo !== undefined &&
			node.data.fileInfo.createDate !== undefined &&
			moment(node.data.fileInfo.createDate).isValid()
		) {
			return moment(node.data.fileInfo.createDate).format(format);
		} else {
			return '';
		}
	}

	createRelationshipDateTemplate(format, node) {
		if (
			node.data !== undefined &&
			node.data.fileInfo !== undefined &&
			node.data.fileInfo.relationshipsDate !== undefined &&
			moment(node.data.fileInfo.relationshipsDate).isValid()
		) {
			return moment(node.data.fileInfo.relationshipsDate).format(format);
		} else {
			return '';
		}
	}

	extractTar(event) {
		untar(event.target.result).then(
			extractedFiles => {
				this.setState({ extractedFiles }, () => this.parseFiles());
			},
			err => {
				console.log(err);
			},
			() => {
				// onProgress
			}
		);
	}

	onBeforeSent(e) {
		for (const pair of e.formData.entries()) {
			const file = pair[1];
			if (window.FileReader) {
				const reader = new FileReader();
				reader.onloadend = this.extractTar.bind(this);
				reader.readAsArrayBuffer(file);
			} else {
				alert("Sorry, your browser doesn't support for preview");
			}
		}
	}

	parseFiles() {
		const { extractedFiles } = this.state;
		const documents = [];
		const metadatas = [];
		const cases = [];
		const parsingErrors = [];
		extractedFiles.forEach(extractedFile => {
			const fileDirectory = extractedFile.name.split(/[\/\\]/);
			if (fileDirectory.length >= 2 && fileDirectory[fileDirectory.length - 1] !== '') {
				switch (fileDirectory[0]) {
					case 'dokumenty':
						extractedFile.extractedName = fileDirectory[fileDirectory.length - 1];
						extractedFile.type = 'Dokument';
						documents[fileDirectory[fileDirectory.length - 1].toUpperCase()] = extractedFile;
						break;
					case 'metadane':
						extractedFile.extractedName = fileDirectory[fileDirectory.length - 1];
						extractedFile.type = 'Metadane';
						metadatas.push(extractedFile);
						break;
					case 'sprawy':
						extractedFile.type = 'Sprawa';
						const content = new TextDecoder().decode(new Uint8Array(extractedFile.buffer));
						let xmlContent;
						xml2js.parseString(content, { explicitArray: false }, (err, result) => {
							xmlContent = result;
						});
						if (xmlContent !== undefined && xmlContent !== null) {
							const title = this.extractTitle(xmlContent);
							extractedFile.extractedName = title ? title : fileDirectory[1];
							extractedFile.originalFileName = fileDirectory[1];
							cases[extractedFile.extractedName] = extractedFile;
						} else {
							const error = `Błąd w strukturze pliku ${extractedFile.name}`;
							if (!parsingErrors.includes(error)) {
								parsingErrors.push(error);
							}
						}
						break;
					default:
						const error = `Błędna struktura paczki, istnieje folder o nazwie ${extractedFile.name}`;
						if (!parsingErrors.includes(error)) {
							parsingErrors.push(error);
						}
						break;
				}
			}
		});
		metadatas.forEach(metadata => {
			const content = new TextDecoder().decode(new Uint8Array(metadata.buffer));
			let xmlContent;
			xml2js.parseString(content, { explicitArray: false }, (err, result) => {
				xmlContent = result;
			});
			let caseFile;
			if (xmlContent !== undefined && xmlContent !== null) {
				if (xmlContent.dokument.grupowanie instanceof Array) {
					xmlContent.dokument.grupowanie.forEach(g => {
						if (g.typ === 'znak sprawy') {
							caseFile = cases[`${g.kod}`.toUpperCase()];
						}
					});
				} else {
					caseFile = cases[`${xmlContent.dokument.grupowanie.kod}`.toUpperCase()];
				}
				if (caseFile !== undefined && caseFile !== null) {
					let caseFiles = caseFile.files;
					if (caseFiles === undefined) {
						caseFiles = [];
					}
					let document;
					if (xmlContent.dokument.identyfikator instanceof Array) {
						xmlContent.dokument.identyfikator.forEach(g => {
							if (g.typ === 'EZD.PUW.NazwaZalacznika') {
								document = documents[g.wartosc.toUpperCase()];
							}
						});
					} else {
						document = documents[xmlContent.dokument.identyfikator.wartosc.toUpperCase()];
					}
					if (document === undefined || document === null) {
						document =
							documents[
								metadata.extractedName.substring(0, metadata.extractedName.length - 4).toUpperCase()
							];
					}
					if (document === undefined || document === null) {
						const fileNameFromMeta = this.extractFileName(xmlContent);
						for (const docId in documents) {
							const sizeFromMeta = this.extractFileSize(xmlContent);
							if (
								metadata.extractedName.toUpperCase().includes(docId) &&
								documents[docId].size === sizeFromMeta
							) {
								document = documents[docId];
								if (fileNameFromMeta !== undefined) {
									document.extractedName = fileNameFromMeta;
								}
								break;
							}
						}
					}
					if (document !== undefined && document !== null) {
						document.metadata = metadata;
						const title = this.extractTitle(xmlContent);
						const createDate = this.extractCreateDate(xmlContent);
						const author = this.extractAuthor(xmlContent);
						const relationGui = this.extractRelationGui(xmlContent);
						const relationshipsDate = this.extractRelationsDate(xmlContent, createDate);
						document.fileInfo = {
							title,
							createDate,
							author,
							relationGui,
							relationshipsDate,
						};
						caseFiles.push(document);
						caseFile.files = caseFiles;
					} else {
						if (xmlContent.dokument.identyfikator instanceof Array) {
							xmlContent.dokument.identyfikator.forEach(g => {
								if (g.typ === 'EZD.PUW.NazwaZalacznika') {
									const error = `Dokument nie istnieje ${g.wartosc}`;
									if (!parsingErrors.includes(error)) {
										parsingErrors.push(error);
									}
								}
							});
						} else {
							const error = `Dokument nie istnieje ${xmlContent.dokument.identyfikator.wartosc}`;
							if (!parsingErrors.includes(error)) {
								parsingErrors.push(error);
							}
						}
					}
				} else {
					const error = `Sprawa nie istnieje ${xmlContent.dokument.grupowanie.kod}`;
					if (!parsingErrors.includes(error)) {
						parsingErrors.push(error);
					}
				}
			} else {
				const error = `Plik metadanych nie jest poprawnym dokumentem XML ${
					metadata.extractFileName
				}`;
				if (!parsingErrors.includes(error)) {
					parsingErrors.push(error);
				}
			}
		});
		const root = [];
		let caseIndex = 0;
		for (const caseId in cases) {
			const caseFile = cases[caseId];
			const content = new TextDecoder().decode(new Uint8Array(caseFile.buffer));
			let xmlContent;
			xml2js.parseString(content, { explicitArray: false }, (err, result) => {
				xmlContent = result;
			});
			if (xmlContent !== undefined && xmlContent !== null) {
				const title = this.extractTitle(xmlContent);
				const createDate = this.extractCreateDate(xmlContent);
				const author = this.extractAuthor(xmlContent);
				const relationGui = this.extractRelationGui(xmlContent);
				const relationshipsDate = this.extractRelationsDate(xmlContent, createDate);
				caseFile.fileInfo = {
					title,
					createDate,
					author,
					relationGui,
					relationshipsDate,
				};
				let children = undefined;
				if (caseFile.files !== undefined && caseFile.files !== null && caseFile.files.length > 0) {
					let documentIndex = 0;
					children = [];
					for (const fileId in caseFile.files) {
						const file = caseFile.files[fileId];
						children.push({ key: `${caseIndex}-${documentIndex}`, data: file });
						documentIndex = documentIndex + 1;
					}
				}
				cases[caseId] = caseFile;
				root.push({ key: caseIndex.toString(), data: caseFile, children });
				caseIndex = caseIndex + 1;
			} else {
				const error = `Błędny plik sprawy ${caseId}`;
				if (!parsingErrors.includes(error)) {
					parsingErrors.push(error);
				}
			}
		}
		if (parsingErrors.length > 0) {
			this.messages.show({
				severity: 'info',
				summary: 'Komunikat',
				detail:
					// eslint-disable-next-line max-len
					'Paczka zawiera błędy. Wizualizator nie wyświetla poprawnie zawartości paczki. Szczegółowe informacje dostępne są w raportach z walidacji.',
				life: 20000,
			});
		}
		this.setState({ treeTableSource: root, treeTableSourceFiltered: root });
	}

	extractFileSize(xmlContent) {
		if (
			xmlContent.dokument.format !== undefined &&
			xmlContent.dokument.format.wielkosc !== undefined
		) {
			return parseInt(xmlContent.dokument.format.wielkosc._);
		} else {
			return undefined;
		}
	}

	extractFileName(xmlContent) {
		if (xmlContent.dokument.identyfikator instanceof Array) {
			xmlContent.dokument.identyfikator.forEach(g => {
				if (g.typ === 'EZD.PUW.NazwaZalacznika') {
					return g.wartosc;
				}
			});
		} else {
			if (
				xmlContent.dokument.identyfikator !== undefined &&
				xmlContent.dokument.identyfikator.wartosc !== undefined
			) {
				return xmlContent.dokument.identyfikator.wartosc;
			} else {
				return undefined;
			}
		}
		return undefined;
	}

	extractTitle(xmlContent) {
		if (
			xmlContent.dokument &&
			xmlContent.dokument.identyfikator !== undefined &&
			xmlContent.dokument.identyfikator.wartosc !== undefined &&
			xmlContent.dokument.identyfikator.wartosc instanceof Array &&
			xmlContent.dokument.identyfikator.wartosc.length > 0
		) {
			return xmlContent.dokument.identyfikator.wartosc[0]._;
		} else if (
			xmlContent.dokument &&
			xmlContent.dokument.identyfikator !== undefined &&
			xmlContent.dokument.identyfikator.wartosc !== undefined &&
			xmlContent.dokument.identyfikator.wartosc instanceof Object
		) {
			return xmlContent.dokument.identyfikator.wartosc._;
		} else if (
			xmlContent.dokument &&
			xmlContent.dokument.identyfikator &&
			xmlContent.dokument.identyfikator.wartosc &&
			xmlContent.dokument.identyfikator.wartosc !== undefined &&
			typeof xmlContent.dokument.identyfikator.wartosc === 'string'
		) {
			return xmlContent.dokument.identyfikator.wartosc;
		} else {
			return '';
		}
	}

	extractTitleForView(xmlContent) {
		if (
			xmlContent.dokument.tytul !== undefined &&
			xmlContent.dokument.tytul.oryginalny !== undefined &&
			xmlContent.dokument.tytul.oryginalny instanceof Array &&
			xmlContent.dokument.tytul.oryginalny.length > 0
		) {
			return xmlContent.dokument.tytul.oryginalny[0]._;
		} else if (
			xmlContent.dokument.tytul !== undefined &&
			xmlContent.dokument.tytul.oryginalny !== undefined &&
			xmlContent.dokument.tytul.oryginalny instanceof Object
		) {
			return xmlContent.dokument.tytul.oryginalny._;
		} else {
			return '';
		}
	}

	extractCreateDate(xmlContent) {
		if (xmlContent.dokument.data !== undefined && xmlContent.dokument.data instanceof Array) {
			for (const data of xmlContent.dokument.data) {
				if (typeof data.typ === 'string') {
					if (
						data.typ !== undefined &&
						data.typ !== null &&
						data.typ.toUpperCase() === 'STWORZONY'
					) {
						return new Date(data.czas);
					} else if (
						data.typ !== undefined &&
						data.typ !== null &&
						data.typ.toUpperCase() === 'DATY SKRAJNE'
					) {
						return new Date(data.od);
					}
				}
			}
		} else if (
			xmlContent.dokument.data !== undefined &&
			xmlContent.dokument.data instanceof Object &&
			xmlContent.dokument.data.typ !== undefined &&
			typeof xmlContent.dokument.data.typ === 'string' &&
			xmlContent.dokument.data.typ.toUpperCase() === 'STWORZONY'
		) {
			return new Date(xmlContent.dokument.data.czas);
		} else if (
			xmlContent.dokument.data !== undefined &&
			xmlContent.dokument.data instanceof Object &&
			xmlContent.dokument.data.typ !== undefined &&
			typeof xmlContent.dokument.data.typ === 'string' &&
			xmlContent.dokument.data.typ.toUpperCase() === 'DATY SKRAJNE'
		) {
			return new Date(xmlContent.dokument.data.od);
		}
	}

	extractRelationsDate(xmlContent, createDate) {
		if (
			xmlContent.dokument.relacja &&
			xmlContent.dokument.relacja.typ &&
			(xmlContent.dokument.relacja.typ.toUpperCase() === 'JESTDEKRETACJĄ' ||
				xmlContent.dokument.relacja.typ.toUpperCase() === 'ODNOSISIEDO' ||
				xmlContent.dokument.relacja.typ.toUpperCase() === 'MADEKRETACJĘ' ||
				xmlContent.dokument.relacja.typ.toUpperCase() === 'ODNOSISIĘDO' ||
				xmlContent.dokument.relacja.typ.toUpperCase() === 'MAODNIESIENIE')
		) {
			return createDate;
		} else if (
			xmlContent.dokument.relacja !== undefined &&
			xmlContent.dokument.relacja instanceof Array
		) {
			return createDate;
		} else return undefined;
	}

	extractRelationGui(xmlContent) {
		const result = [];
		if (xmlContent.dokument.relacja !== undefined && xmlContent.dokument.relacja instanceof Array) {
			for (const relacja of xmlContent.dokument.relacja) {
				result.push({
					typ: relacja.typ,
					identyfikator: {
						typ: relacja.identyfikator.typ,
						wartosc: relacja.identyfikator.wartosc,
					},
				});
			}
		} else if (
			xmlContent.dokument.relacja !== undefined &&
			xmlContent.dokument.relacja instanceof Object
		) {
			result.push({
				typ: xmlContent.dokument.relacja.typ,
				identyfikator: {
					typ: xmlContent.dokument.relacja.identyfikator.typ,
					wartosc: xmlContent.dokument.relacja.identyfikator.wartosc,
				},
			});
		}
		return result;
	}

	extractAuthor(xmlContent) {
		let tworca = undefined;
		if (xmlContent.dokument.tworca !== undefined && xmlContent.dokument.tworca instanceof Array) {
			for (const tworcaFromXml of xmlContent.dokument.tworca) {
				if (
					typeof tworcaFromXml.funkcja === 'string' &&
					(tworcaFromXml.funkcja.toUpperCase() === 'STWORZYL' ||
						tworcaFromXml.funkcja.toUpperCase() === 'STWORZYŁ' ||
						tworcaFromXml.funkcja.toUpperCase() === 'UTWORZYL' ||
						tworcaFromXml.funkcja.toUpperCase() === 'UTWORZYŁ')
				) {
					tworca = tworcaFromXml;
				}
			}
		} else if (
			xmlContent.dokument.tworca !== undefined &&
			xmlContent.dokument.tworca instanceof Object &&
			typeof xmlContent.dokument.tworca.funkcja === 'string' &&
			(xmlContent.dokument.tworca.funkcja.toUpperCase() === 'STWORZYL' ||
				xmlContent.dokument.tworca.funkcja.toUpperCase() === 'STWORZYŁ' ||
				xmlContent.dokument.tworca.funkcja.toUpperCase() === 'UTWORZYL' ||
				xmlContent.dokument.tworca.funkcja.toUpperCase() === 'UTWORZYŁ')
		) {
			tworca = xmlContent.dokument.tworca;
		}
		if (tworca !== undefined) {
			if (tworca.podmiot.osoba !== undefined && tworca.podmiot.osoba !== null) {
				return `${tworca.podmiot.osoba.imie} ${tworca.podmiot.osoba.nazwisko}`;
			} else if (tworca.podmiot.instytucja !== undefined && tworca.podmiot.instytucja !== null) {
				if (
					tworca.podmiot.instytucja.jednostka !== undefined &&
					tworca.podmiot.instytucja.jednostka !== null
				) {
					if (
						tworca.podmiot.instytucja.jednostka.pracownik !== undefined &&
						tworca.podmiot.instytucja.jednostka.pracownik !== null &&
						tworca.podmiot.instytucja.jednostka.pracownik.imie !== undefined &&
						tworca.podmiot.instytucja.jednostka.pracownik.imie !== null &&
						tworca.podmiot.instytucja.jednostka.pracownik.imie !== ''
					) {
						// eslint-disable-next-line max-len
						return `${tworca.podmiot.instytucja.jednostka.pracownik.imie} ${
							tworca.podmiot.instytucja.jednostka.pracownik.nazwisko
						}`;
					} else {
						return tworca.podmiot.instytucja.jednostka.nazwa;
					}
				} else {
					return tworca.podmiot.instytucja.nazwa;
				}
			}
		}
	}

	componentDidMount() {
		super.componentDidMount();
		this.readTreeTableSourceFromStorage();
	}

	readTreeTableSourceFromStorage() {
		const treeTableSourceFromStorage = this.readCookie('visualisation-standalone-tree-source');
		const treeTableSourceFilteredFromStorage = this.readCookie(
			'visualisation-standalone-tree-source-filtered'
		);
		const expandedKeysFromStorage = this.readCookie('visualisation-standalone-tree-expanded-keys');
		if (
			this.props.loadFromStorage &&
			treeTableSourceFromStorage !== undefined &&
			treeTableSourceFromStorage !== null
		) {
			const treeTableSource = JSON.parse(treeTableSourceFromStorage);
			let expandedKeys = [];
			if (expandedKeysFromStorage !== undefined && expandedKeysFromStorage !== null) {
				expandedKeys = JSON.parse(expandedKeysFromStorage);
			}
			let treeTableSourceFiltered = [];
			if (
				treeTableSourceFilteredFromStorage !== undefined &&
				treeTableSourceFilteredFromStorage !== null
			) {
				treeTableSourceFiltered = JSON.parse(treeTableSourceFilteredFromStorage);
			} else {
				treeTableSourceFiltered = treeTableSource;
			}
			this.setState({ treeTableSource, expandedKeys, treeTableSourceFiltered });
		}
		this.removeCookie('visualisation-standalone-tree-source');
		this.removeCookie('visualisation-standalone-tree-source-filtered');
		this.removeCookie('visualisation-standalone-tree-expanded-keys');
	}

	handleAfterFilesChange() {
		if (this.state.packageTar !== undefined && this.state.packageTar !== null) {
			if (window.FileReader) {
				const reader = new FileReader();
				reader.onloadend = this.extractTar.bind(this);
				reader.readAsArrayBuffer(this.state.packageTar);
				this.setState({ files: [this.state.packageTar] });
			} else {
				alert("Sorry, your browser doesn't support for preview");
			}
		}
	}

	handleFileRemove() {
		this.setState({
			packageTar: undefined,
			files: [],
			loading: true,
			extractedFiles: [],
			expandedKeys: [],
			treeTableSource: [],
			treeTableSourceFiltered: [],
			criteria: {
				searchDocumentName: undefined,
				documentDateFrom: undefined,
				documentDateTo: undefined,
			},
		});
	}

	prepareHeaderItems() {
		return [
			{
				label: 'Wczytaj paczkę archiwalną',
				type: 'LABEL',
				className: '',
			},
		];
	}

	renderXmlDialog() {}

	render() {
		const cols = [
			{
				field: 'title',
				header: 'Sprawa',
				body: this.caseTemplate,
				expander: true,
			},
			{ header: 'Dokumenty', body: this.documentTemplate },
			{ header: 'Załączniki', body: this.attachmentTemplate },
			{
				header: 'Data utworzenia',
				body: this.createDateTemplate.bind(this, 'DD-MM-YYYY HH:mm'),
			},
			{
				header: 'Autor',
				body: this.authorTemplate,
			},
			{ header: 'Relacje', body: this.relationTemplate },
			{
				field: 'relationshipsDate',
				header: 'Data ustanowienia relacji',
				body: this.createRelationshipDateTemplate.bind(this, 'DD-MM-YYYY HH:mm'),
			},
			{ header: 'Metadane', body: this.actionTemplate },
		];
		const dynamicColumns = cols.map((col, i) => {
			return (
				<Column
					key={`${i}-${col.field !== undefined ? col.field : col.header}`}
					columnKey={`${i}-${col.field !== undefined ? col.field : col.header}`}
					field={col.field}
					header={col.header}
					body={col.body}
					expander={col.expander}
				/>
			);
		});
		return (
			<div>
				<CustomMessages ref={el => (this.messages = el)} />
				<Card header={this.renderHeader()}>
					{this.state.xmlEditorVisible ? this.renderXmlDialog() : null}
					<DivContainer colClass="row">
						<CustomLabel label="Paczka archiwalna" colClass="col-md-12" />
						<InputFileChooseComponent
							accepts={['.tar']}
							colClass="col-md-12"
							label="Paczka archiwalna"
							showLabel={false}
							file={this.state.packageTar}
							id="package-file"
							maxFileSize={1000000000}
							maxFiles={1}
							multiple={false}
							name="packageTar"
							onChange={this.handleChange}
							onAfterStateChange={this.handleAfterFilesChange}
							onAfterRemove={this.handleFileRemove}
							stateField=""
							uploadMaxSizeParamName="PACK_ATT_MAX"
							baseRestApiUrl={this.props.baseRestApiUrl}
						>
							Upuść plik tutaj lub kliknij aby przesłać
						</InputFileChooseComponent>
						<DivContainer colClass="col-xl-12">
							{this.addSearchArea()}
							<DivContainer colClass="row">
								<TreeTable
									key="treeTable"
									value={this.state.treeTableSourceFiltered}
									expandedKeys={this.state.expandedKeys}
									onToggle={e => this.setState({ expandedKeys: e.value })}
									emptyMessage="Brak rekordów do wyświetlenia"
								>
									{dynamicColumns}
								</TreeTable>
							</DivContainer>
						</DivContainer>
					</DivContainer>
				</Card>
			</div>
		);
	}
}

BaseVisualisationStandaloneContainer.defaultProps = {};

BaseVisualisationStandaloneContainer.propTypes = {
	loadFromStorage: PropTypes.bool.isRequired,
};

export default BaseVisualisationStandaloneContainer;
