/* eslint-disable react/jsx-max-props-per-line */
/* eslint-disable react/destructuring-assignment */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import Quill from 'quill';
import 'quill/dist/quill.snow.css';
import 'quill/dist/quill.bubble.css';

export class Editor extends Component {
	componentDidMount() {
		this.quill = new Quill(this.editorElement, {
			modules: {
				toolbar: this.toolbarElement,
				...this.props.modules,
			},
			placeholder: this.props.placeholder,
			readOnly: this.props.readOnly,
			theme: this.props.theme,
			formats: this.props.formats,
		});

		if (this.props.value) {
			this.quill.pasteHTML(this.props.value);
		}

		this.quill.on('text-change', (delta, source) => {
			let html = this.editorElement.children[0].innerHTML;
			const text = this.quill.getText();
			if (html === '<p><br></p>') {
				html = null;
			}

			if (this.props.onTextChange) {
				this.props.onTextChange({
					htmlValue: html,
					textValue: text,
					delta,
					source,
				});
			}
		});

		this.quill.on('selection-change', (range, oldRange, source) => {
			if (this.props.onSelectionChange) {
				this.props.onSelectionChange({
					range,
					oldRange,
					source,
				});
			}
		});
	}

	componentDidUpdate(prevProps) {
		if (this.props.value !== prevProps.value && this.quill && !this.quill.hasFocus()) {
			if (this.props.value) this.quill.pasteHTML(this.props.value);
			else this.quill.setText('');
		}
	}

	render() {
		const containerClass = classNames('p-component p-editor-container', this.props.className);
		let toolbarHeader = null;

		if (this.props.headerTemplate) {
			toolbarHeader = (
				<div ref={el => (this.toolbarElement = el)} className="p-editor-toolbar">
					{this.props.headerTemplate}
				</div>
			);
		} else {
			toolbarHeader = (
				<div ref={el => (this.toolbarElement = el)} className="p-editor-toolbar">
					<span className="ql-formats">
						<select className="ql-header" defaultValue="0">
							<option value="1">Heading</option>
							<option value="2">Subheading</option>
							<option value="0">Normal</option>
						</select>
						<select className="ql-font">
							<option />
							<option value="serif" />
							<option value="monospace" />
						</select>
					</span>
					<span className="ql-formats">
						<button type="button" className="ql-bold" aria-label="Bold" />
						<button type="button" className="ql-italic" aria-label="Italic" />
						<button type="button" className="ql-underline" aria-label="Underline" />
					</span>
					<span className="ql-formats">
						<select className="ql-color" />
						<select className="ql-background" />
					</span>
					<span className="ql-formats">
						<button type="button" className="ql-list" value="ordered" aria-label="Ordered List" />
						<button type="button" className="ql-list" value="bullet" aria-label="Unordered List" />
						<select className="ql-align">
							<option defaultValue />
							<option value="center" />
							<option value="right" />
							<option value="justify" />
						</select>
					</span>
					<span className="ql-formats">
						<button type="button" className="ql-link" aria-label="Insert Link" />
						<button type="button" className="ql-image" aria-label="Insert Image" />
						<button type="button" className="ql-code-block" aria-label="Insert Code Block" />
					</span>
					<span className="ql-formats">
						<button type="button" className="ql-clean" aria-label="Remove Styles" />
					</span>
				</div>
			);
		}

		const content = (
			<div
				ref={el => (this.editorElement = el)}
				className="p-editor-content"
				style={this.props.style}
			/>
		);

		return (
			<div id={this.props.id} className={containerClass}>
				{toolbarHeader}
				{content}
			</div>
		);
	}
}

Editor.defaultProps = {
	id: null,
	value: null,
	style: null,
	className: null,
	placeholder: null,
	readOnly: false,
	modules: null,
	formats: null,
	theme: 'snow',
	headerTemplate: null,
	onTextChange: null,
	onSelectionChange: null,
};

Editor.propTypes = {
	ariaDescribedBy: PropTypes.string,
	ariaLabel: PropTypes.string,
	ariaLabelledBy: PropTypes.string,
	className: PropTypes.string,
	formats: PropTypes.array,
	headerTemplate: PropTypes.any,
	id: PropTypes.string,
	modules: PropTypes.object,
	onSelectionChange: PropTypes.func,
	onTextChange: PropTypes.func,
	placeholder: PropTypes.string,
	readOnly: PropTypes.bool,
	required: PropTypes.bool,
	style: PropTypes.object,
	theme: PropTypes.string,
	value: PropTypes.string,
};

export default Editor;
